/*
 * $Id: Memo.cc,v 1.7 2003/03/27 03:31:09 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "qualresUtility.h"
#include "support.h"
#include "Memo.h"

void memoUnmark_cb(GtkWidget *, gpointer);

/** Create empty Memo.

    \param t is the Project tag table
 */
Memo::Memo(GtkTextTagTable *t)
{
  init(t);
}

/** Initialize the object.

    \param t is the Project tag table.

    This method creates a GtkTextBuffer.
*/
void
Memo::init(GtkTextTagTable *t)
{
  objType = QualResObject::MEMO;
  objName = "Memo";
  xmlTag = MEMO_TAG;
  projTagTable = t;
  textBuffer = gtk_text_buffer_new(projTagTable);
  createWindow(NULL);
  setEditable(TRUE);
}

/** Destructor.
 */
Memo::~Memo()
{}

/** Create the window to code and view a Memo.

    \param parent is the parent window

 */
void
Memo::createWindow(GtkWidget *parent)
{
  /* Menus */
  static GnomeUIInfo fileMenu [] = {
    {GNOME_APP_UI_ITEM,
     N_("Import from file"),
     N_("Import text from a file to the cursor position"),
     (gpointer) documentImport_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CONVERT,
     0, (GdkModifierType) 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Export Memo"),
     N_("Export the Memo to a file"),
     (gpointer) docCatExport_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CONVERT,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Print preview"),
     N_("Print preview"),
     (gpointer) printPreview_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_PRINT_PREVIEW,
     0, (GdkModifierType) 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Print"),
     N_("Print the Memo"),
     (gpointer) print_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_PRINT,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Close window"),
     N_("Close the Memo window"),
     (gpointer) close_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CLOSE,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_END
  };

  static GnomeUIInfo xrefMenu [] = {
    {GNOME_APP_UI_ITEM,
     N_("Select Cross-reference"),
     N_("Select the Cross-reference for cross-referencing text"),
     (gpointer) selectCodeCategoryList_cb, NULL, NULL,
     GNOME_APP_PIXMAP_NONE,
     NULL,
     0, (GdkModifierType) 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Open Selected Cross-reference"),
     N_("Open the Selected Cross-reference"),
     (gpointer) codeViewSelectedCat_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_OPEN,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_END
  };

  static GnomeUIInfo help_menu [] = {
    GNOMEUIINFO_HELP ("Document"),
  
    GNOMEUIINFO_END
  };

  static GnomeUIInfo menu [] = {
    { GNOME_APP_UI_SUBTREE_STOCK, N_("File"), NULL, fileMenu, NULL, NULL,
      (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

    { GNOME_APP_UI_SUBTREE_STOCK, N_("Category"), NULL, xrefMenu, NULL, NULL,
      (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

    GNOMEUIINFO_MENU_HELP_TREE(help_menu),
    GNOMEUIINFO_END
  };

  GtkWidget *status, *vbox, *hbox, *frame;
  GtkWidget *scrolledView, *view;
  GtkWidget *button;
  PangoFontDescription *pango;
  
  string sProp;

  // Create widgets.
  mainWindow = gnome_app_new("Memo", _("Memo"));
  gtk_window_set_default_size(GTK_WINDOW(mainWindow),
			      MEMO_XSIZE,
			      MEMO_YSIZE);

  status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);

  textView = gtk_text_view_new_with_buffer(textBuffer);
  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textView),
			      GTK_WRAP_WORD);
  gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(textView),
				       GTK_TEXT_WINDOW_LEFT,
				       1);
  g_signal_connect(G_OBJECT(textView),
		   "expose-event",
		   G_CALLBACK(text_view_line_numbers_expose_cb),
		   textBuffer);
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_signal_connect(GTK_OBJECT(mainWindow), "delete-event",
		     GTK_SIGNAL_FUNC(hide_cb), this);

  vbox = gtk_vbox_new(FALSE, 5);

  editableLabel = gtk_label_new(_("Editable"));
  gtk_box_pack_start(GTK_BOX(vbox),
		     editableLabel,
		     FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 5);

  button = gtk_button_new_with_label("Select");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(selectCodeCategoryList_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(hbox),
		     gtk_label_new(_("Selected Cross-reference:  ")),
		     FALSE, FALSE, 0);

  selectedCatBuffer = gtk_text_buffer_new
    (gtk_text_buffer_get_tag_table(textBuffer));
  view = gtk_text_view_new_with_buffer(selectedCatBuffer);
  g_signal_connect(G_OBJECT(view),
		   "button-press-event",
		   G_CALLBACK(selectedCatViewButtonPress_cb),
		   this);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(view),
			     FALSE);

  gtk_box_pack_start(GTK_BOX(hbox),
		     view,
		     FALSE, FALSE, 0);

  button = gtk_button_new_with_label("Cross-ref");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(documentCode_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  button = gtk_button_new_with_label("Undo Cross-ref");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(documentUncode_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  button = gtk_button_new_with_label("Unmark text");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(memoUnmark_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(vbox),
		     hbox,
		     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_hseparator_new(),
		     FALSE, FALSE, 0);

  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), textView);

  gtk_box_pack_start(GTK_BOX(vbox),
		     scrolledView,
		     TRUE, TRUE, 0);

  // Add to application window.
  gnome_app_set_contents(GNOME_APP(mainWindow), frame);
  gnome_app_set_statusbar(GNOME_APP(mainWindow), status);
  gnome_app_create_menus_with_data(GNOME_APP(mainWindow), menu, this);
  gnome_app_install_menu_hints(GNOME_APP(mainWindow), menu);

  // Add the widgets.
  gtk_container_add(GTK_CONTAINER(frame), vbox);
}

/** Get the Component name labeled with "Memo:  ".

    \returns left header of print output (free with g_free)
**/
gchar *
Memo::getLabeledName() const
{
  return(g_strconcat("Memo:  ", (getPrettyName()).c_str(), NULL));
}

/** Unmark text.
 */
void
Memo::unmarkText()
{
  GtkTextIter start, end;
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(textBuffer),
			     &start,
			     &end);

  gtk_text_buffer_remove_all_tags(GTK_TEXT_BUFFER(textBuffer),
				  &start,
                                  &end);
}

// Callbacks.
/** Unmark text in a Memo.
*/
void
memoUnmark_cb(GtkWidget *widget, gpointer data)
{
  Memo *mem = (Memo *) data;

  mem->unmarkText();
}
