/*
 * $Id: Memo.h,v 1.5 2003/03/10 03:03:17 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef MEMO_H
#define MEMO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <string>
#include "Document.h"


/** Class for a Memo

    This class implements the methods for a Memo.

    All access functions of this and derived classes return a pointer
    to const.  For example, getName returns (const string *), pointing
    to the name field.  The object is responsible for this memory; the
    calling program should never free the memory pointed to by this
    pointer.
*/
class Memo : public Document
{
public:
  Memo(GtkTextTagTable *);
  virtual ~Memo();

 // Printing methods.
#if 0
  virtual gint getPrintLineCount();
  virtual gchar *getTextToPrint() const;
  virtual gboolean hasSelectedText();
  virtual gchar *getSelectedTextToPrint() const;
  virtual gint getSelectedTextToPrintLineNum();
  virtual gint printLineNumbers()   ///< How often to number lines.
    { return(1); }
#endif
  virtual gchar *getLabeledName() const;

  // GUI methods.
  virtual void createWindow(GtkWidget *);
  /// No Category (or Cross-reference) labels.
  virtual void addCategoryLabel(Category *c, GtkTextIter *i) {}
  /// No Category (or Cross-reference) labels.
  virtual void removeCategoryLabel(Category *, GtkTextIter *) {}
  /// No Category (or Cross-reference) labels.
  virtual void adjustCategoryLabels() {}

  void unmarkText();

private:
  void init(GtkTextTagTable *);
};

#endif
