/*
 * $Id: PrintObject.h,v 1.1.1.1 2002/12/31 03:12:45 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef PRINT_OBJECT_H
#define PRINT_OBJECT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <gtk/gtk.h>
#include <gnome.h>

/** Base class for an object that can output to a printer.
*/
class PrintObject
{
public:
  PrintObject() { parent = NULL; } ///< Constructor.

  void print();
  void printPreview();

  /// Load text to print.
  virtual void loadTextToPrint(string &b) { buffer = b; }

  // Methods to interface to the print functions.
  /// True if there's selected text.
  virtual gboolean hasSelectedText() { return(FALSE); }

  /** Get the line number of the first line of selected text to print.

      This is not invoked because there is no selected text.
  */
  virtual gint getSelectedTextToPrintLineNum() { return(1); }

  /// How often to number lines (0 means never).
  virtual gint printLineNumbers() { return(0); }

  /** Get the number of lines to print.

      This is not invoked because line numbers are not printed.
  */
  virtual gint getPrintLineCount() { return(1); }

 /// Get the text to print.
  virtual gchar *getTextToPrint() const
    {
      return(g_strdup_printf(buffer.c_str()));
    }

 /// Get the Selected text to print.
  virtual gchar *getSelectedTextToPrint() const
    {
      return(g_strdup(buffer.c_str()));
    }

  /// Load the title of the print page.
  void setLabeledName(string l) { labeledName = l; }

  /// Get the title of the print page.
  virtual gchar *getLabeledName() const {g_strdup(labeledName.c_str()); }

  /// Load the address of the parent window pointer.
  void setParentWindow(GtkWidget **p) { g_assert(p != NULL); parent = p; }

  /// Get the parent window.
  GtkWidget *getParentWindow() { g_assert(parent != NULL); return(*parent); }

private:
  GtkWidget **parent; ///< Pointer to parent window of print dialog.
  string buffer; ///< Text to print.
  string labeledName; ///< Labeled name.
};

// Callbacks.
void printPreview_cb(GtkWidget *, gpointer);
void print_cb(GtkWidget*, gpointer);

#endif
