/*
 * $Id: Project.cc,v 1.25 2003/04/16 14:11:47 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>

extern const char *const sys_errlist[];
extern int errno;

#include <fstream>
#include <strstream>
#include <stack>
#include <libgdome/gdome.h>
#include "qualresUtility.h"
#include "PrintObject.h"
#include "Project.h"

// Callbacks.
static void saveProject_cb(GtkWidget *, gpointer);
static void saveProjectAs_cb(GtkWidget *, gpointer);
static void closeProject_cb(GtkWidget *, gpointer);
static void restoreProject_cb(GtkWidget *, gpointer);
static void changeProjectProperties_cb(GtkWidget*, gpointer);
static void quit_cb(GtkWidget*, gpointer);
static void printDocumentList_cb(GtkWidget *, gpointer);
static void printCategoryList_cb(GtkWidget *, gpointer);
static void printAllDocuments_cb(GtkWidget *, gpointer);
static void printAllCategories_cb(GtkWidget *, gpointer);
static void exportDocumentList_cb(GtkWidget *, gpointer);
static void exportCategoryList_cb(GtkWidget *, gpointer);

static void codeViewDocument_cb(GtkWidget*, gpointer);
static void newSiblingDocument_cb(GtkWidget*, gpointer);
static void newChildDocument_cb(GtkWidget*, gpointer);
static void cutDocument_cb(GtkWidget*, gpointer);
static void deleteDocument_cb(GtkWidget*, gpointer);
static void pasteSiblingDocument_cb(GtkWidget*, gpointer);
static void pasteChildDocument_cb(GtkWidget*, gpointer);
static void changeDocumentProperties_cb(GtkWidget*, gpointer);

static void codeViewCategory_cb(GtkWidget*, gpointer);
static void newChildCategory_cb(GtkWidget*, gpointer);
static void newSiblingCategory_cb(GtkWidget*, gpointer);
static void cutCategory_cb(GtkWidget*, gpointer);
static void deleteCategory_cb(GtkWidget*, gpointer);
static void pasteChildCategory_cb(GtkWidget*, gpointer);
static void pasteSiblingCategory_cb(GtkWidget*, gpointer);
static void changeCategoryProperties_cb(GtkWidget*, gpointer);

static void updateViewSet_cb(GtkWidget*, gpointer);
static void newChildSet_cb(GtkWidget*, gpointer);
static void newSiblingSet_cb(GtkWidget*, gpointer);
static void cutSet_cb(GtkWidget*, gpointer);
static void deleteSet_cb(GtkWidget*, gpointer);
static void pasteChildSet_cb(GtkWidget*, gpointer);
static void pasteSiblingSet_cb(GtkWidget*, gpointer);
static void changeSetProperties_cb(GtkWidget*, gpointer);

static void codeViewMemo_cb(GtkWidget*, gpointer);
static void newSiblingMemo_cb(GtkWidget*, gpointer);
static void newChildMemo_cb(GtkWidget*, gpointer);
static void cutMemo_cb(GtkWidget*, gpointer);
static void deleteMemo_cb(GtkWidget*, gpointer);
static void pasteSiblingMemo_cb(GtkWidget*, gpointer);
static void pasteChildMemo_cb(GtkWidget*, gpointer);
static void changeMemoProperties_cb(GtkWidget*, gpointer);

static void codeViewXref_cb(GtkWidget*, gpointer);
static void newSiblingXref_cb(GtkWidget*, gpointer);
static void newChildXref_cb(GtkWidget*, gpointer);
static void cutXref_cb(GtkWidget*, gpointer);
static void deleteXref_cb(GtkWidget*, gpointer);
static void pasteSiblingXref_cb(GtkWidget*, gpointer);
static void pasteChildXref_cb(GtkWidget*, gpointer);
static void changeXrefProperties_cb(GtkWidget*, gpointer);

static void showMemoXrefWindow_cb(GtkWidget*, gpointer);
static void hideMemoXrefWindow_cb(GtkWidget*, gpointer);

static void showSearchWindow_cb(GtkWidget*, gpointer);
static void hideSearchWindow_cb(GtkWidget*, gpointer);
static void selectSearchCategory_cb(GtkWidget*, gpointer);
gboolean searchCatListButtonPress_cb(GtkWidget *, GdkEventButton *, gpointer);
static void search_cb(GtkWidget*, gpointer);

static void showMergeCatWindow_cb(GtkWidget*, gpointer);
gboolean mergeCatMerge_cb(GtkWidget *, gpointer);
gboolean mergeCatDestViewButtonPress_cb(GtkWidget *,
					GdkEventButton *,
					gpointer);
void mergeCatDestListSelect_cb(GtkTreeView *, gpointer);


// Narrative Analysis callbacks.
static void analyzeNarrativeDocument_cb(GtkWidget*, gpointer);
static void newSiblingNarrativeDocument_cb(GtkWidget*, gpointer);
static void newChildNarrativeDocument_cb(GtkWidget*, gpointer);
static void cutNarrativeDocument_cb(GtkWidget*, gpointer);
static void deleteNarrativeDocument_cb(GtkWidget*, gpointer);
static void pasteSiblingNarrativeDocument_cb(GtkWidget*, gpointer);
static void pasteChildNarrativeDocument_cb(GtkWidget*, gpointer);
static void changeNarrativeDocumentProperties_cb(GtkWidget*, gpointer);

// Frame callbacks.
static void newSiblingFrame_cb(GtkWidget*, gpointer);
static void newChildFrame_cb(GtkWidget*, gpointer);
static void cutFrame_cb(GtkWidget*, gpointer);
static void deleteFrame_cb(GtkWidget*, gpointer);
static void pasteSiblingFrame_cb(GtkWidget*, gpointer);
static void pasteChildFrame_cb(GtkWidget*, gpointer);
static void changeFrameProperties_cb(GtkWidget*, gpointer);

static void showNarrativeWindow_cb(GtkWidget*, gpointer);
static void hideNarrativeWindow_cb(GtkWidget*, gpointer);

static void projDestroy_cb(GtkWidget*, gpointer);
static void about_cb(GtkWidget*, gpointer);

static gboolean buttonPress_cb(GtkWidget *, GdkEventButton *, gpointer);

// Autosave callbacks.
static gboolean projectTimeout_cb(gpointer);
static gboolean projectIdle_cb(gpointer);

// Main window menus.
static GnomeUIInfo fileMenu [] = {

  {GNOME_APP_UI_ITEM,
   N_("Open Project"),
   N_("Open an existing Project"),
   (gpointer) restoreProject_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_OPEN,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Project properties"),
   N_("Change Project name, etc"),
   (gpointer) changeProjectProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Save Project"),
   N_("Save the current Project"),
   (gpointer) saveProject_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_SAVE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Save Project as..."),
   N_("Save the current Project to a new file"),
   (gpointer) saveProjectAs_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_SAVE_AS,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Close Project"),
   N_("Save and close the current Project"),
   (gpointer) closeProject_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_CLOSE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Print the Document list"),
   N_("Print the names and descriptions of all the Documents"),
   (gpointer) printDocumentList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Print the Category list"),
   N_("Print the names and descriptions of all the Categories"),
   (gpointer) printCategoryList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Export the Document list"),
   N_("Export the names and descriptions of all the Documents"),
   (gpointer) exportDocumentList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_CONVERT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Export the Category list"),
   N_("Export the names and descriptions of all the Categories"),
   (gpointer) exportCategoryList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_CONVERT,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Print all the Documents"),
   N_("Print all the Documents"),
   (gpointer) printAllDocuments_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Print all the Categories"),
   N_("Print all the Categories"),
   (gpointer) printAllCategories_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  GNOMEUIINFO_MENU_EXIT_ITEM(projDestroy_cb, NULL),

  GNOMEUIINFO_END
};

static GnomeUIInfo docMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Open Documents"),
   N_("Code or view selected Documents"),
   (gpointer) codeViewDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_OPEN,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New sibling Document"),
   N_("Create a new Document that is a sibling of the selected Document"),
   (gpointer) newSiblingDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child Document"),
   N_("Create a new Document that is a child of the selected Document"),
   (gpointer) newChildDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Documents"),
   N_("Cut the selected Documents (you can paste them later)"),
   (gpointer) cutDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Documents"),
   N_("Delete the selected Documents"),
   (gpointer) deleteDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Documents as siblings"),
   N_("Paste preiviously cut Documents as siblings"),
   (gpointer) pasteSiblingDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Documents as children"),
   N_("Paste preivously cut Documents as children"),
   (gpointer) pasteChildDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Filter Documents"),
   N_("Select which Documents to Display"),
   (gpointer) changeDocumentProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Unfilter Documents"),
   N_("Display all Documents"),
   (gpointer) changeDocumentProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Document properties"),
   N_("Change Document name, etc"),
   (gpointer) changeDocumentProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo catMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Open Categories"),
   N_("Code or view selected Categories"),
   (gpointer) codeViewCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_OPEN,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New sibling Category"),
   N_("Create a new Category that is a sibling of the selected Category"),
   (gpointer) newSiblingCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child Category"),
   N_("Create a new Category that is a child of the selected Category"),
   (gpointer) newChildCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Categories"),
   N_("Cut selected Categories (you can paste them later)"),
   (gpointer) cutCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Categories"),
   N_("Delete selected Categories"),
   (gpointer) deleteCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Categories as siblings"),
   N_("Paste previously cut Categories as siblings"),
   (gpointer) pasteSiblingCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Categories as children"),
   N_("Paste preivously cut Categories as children"),
   (gpointer) pasteChildCategory_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Merge Categories"),
   N_("Merge one or more Categories to a single Category"),
   (gpointer) showMergeCatWindow_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Category properties"),
   N_("Change Category name, etc"),
   (gpointer) changeCategoryProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo setMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Open Sets"),
   N_("Update or show members of the selected Sets"),
   (gpointer) updateViewSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_OPEN,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New sibling Set"),
   N_("Create a new Set that is a sibling of the selected Set"),
   (gpointer) newSiblingSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child Set"),
   N_("Create a new Set that is a child of the selected Set"),
   (gpointer) newChildSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Sets"),
   N_("Cut the selected Sets (you can paste them later)"),
   (gpointer) cutSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Sets"),
   N_("Delete the selected Sets"),
   (gpointer) deleteSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Sets as sibling"),
   N_("Paste preivously cut Sets as siblings"),
   (gpointer) pasteSiblingSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Sets as child"),
   N_("Paste preivously cut Sets as children"),
   (gpointer) pasteChildSet_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Set properties"),
   N_("Change Set name, etc"),
   (gpointer) changeSetProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo searchMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Search"),
   N_("Search for text or Coded Text"),
   (gpointer) showSearchWindow_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo memoMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Memos"),
   N_("Open the Memo/Cross-reference window"),
   (gpointer) showMemoXrefWindow_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo analysisMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Narrative Analysis"),
   N_("Open the Narrative Analysis window"),
   (gpointer) showNarrativeWindow_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo helpMenu [] = {
  GNOMEUIINFO_HELP(PACKAGE),
  
  GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
  
  GNOMEUIINFO_END
};

static GnomeUIInfo fullMenu [] = {
  { GNOME_APP_UI_SUBTREE_STOCK, N_("File"), NULL, fileMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Document"), NULL, docMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Category"), NULL, catMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Set"), NULL, setMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Search"), NULL, searchMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Memos"), NULL, memoMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Analysis"), NULL, analysisMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  GNOMEUIINFO_MENU_HELP_TREE(helpMenu),
  GNOMEUIINFO_END
};

// Memo window menus.
static GnomeUIInfo memoFileMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Print the Memo list"),
   N_("Print the names and descriptions of all the Memos"),
   (gpointer) printDocumentList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Print the Cross-reference list"),
   N_("Print the names and descriptions of all the Cross-references"),
   (gpointer) printCategoryList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Close window"),
     N_("Close the window"),
     (gpointer) hideMemoXrefWindow_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CLOSE,
     0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo memoMemoMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Open Memos"),
   N_("Cross-reference or view selected Memos"),
   (gpointer) codeViewMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New sibling Memo"),
   N_("Create a new Memo that is a sibling of the selected Memo"),
   (gpointer) newSiblingMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child Memo"),
   N_("Create a new Memo that is a child of the selected Memo"),
   (gpointer) newChildMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Memos"),
   N_("Cut the selected Memos (you can paste them later)"),
   (gpointer) cutMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Memos"),
   N_("Delete the selected Memos"),
   (gpointer) deleteMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Memos as siblings"),
   N_("Paste preiviously cut Memos as siblings"),
   (gpointer) pasteSiblingMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Memos as children"),
   N_("Paste preivously cut Memo as children"),
   (gpointer) pasteChildMemo_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Memo properties"),
   N_("Change Memo name, etc"),
   (gpointer) changeMemoProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo memoXrefMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Open Cross-references"),
   N_("Code or view selected Cross-references"),
   (gpointer) codeViewXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New sibling Cross-reference"),
   N_("Create a new Cross-reference that is a sibling of the selected "
      "Cross-reference"),
   (gpointer) newSiblingXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child Cross-reference"),
   N_("Create a new Cross-reference that is a child of the selected "
      "Cross-reference"),
   (gpointer) newChildXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Cross-references"),
   N_("Cut selected Cross-references (you can paste them later)"),
   (gpointer) cutXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Cross-references"),
   N_("Delete selected Cross-references"),
   (gpointer) deleteXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Cross-references as siblings"),
   N_("Paste preiviously cut Cross-references as siblings"),
   (gpointer) pasteSiblingXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Cross-references as children"),
   N_("Paste preivously cut Cross-references as children"),
   (gpointer) pasteChildXref_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Cross-reference properties"),
   N_("Change Cross-reference name, etc"),
   (gpointer) changeXrefProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo memoMemoXrefMenu [] = {
  { GNOME_APP_UI_SUBTREE_STOCK, N_("File"), NULL, memoFileMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Memo"), NULL, memoMemoMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Cross-Reference"), NULL, memoXrefMenu,
    NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  GNOMEUIINFO_MENU_HELP_TREE(helpMenu),
  GNOMEUIINFO_END
};

// Narrative Analysis window menus.
static GnomeUIInfo narrativeFileMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Print the Narrative Document list"),
   N_("Print the names and descriptions of all the Narrative "
      "Analysis Documents"),
   (gpointer) printDocumentList_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PRINT,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Close window"),
     N_("Close the window"),
     (gpointer) hideNarrativeWindow_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CLOSE,
     0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo narrativeDocumentMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("Open Narrative Document"),
   N_("Analyze selected Narrative Documents"),
   (gpointer) analyzeNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_NONE,
   NULL,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New sibling Narrative Document"),
   N_("Create a new Narrative Document that is a sibling of the "
      "selected Narrative Document"),
   (gpointer) newSiblingNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child NarrativeDocument"),
   N_("Create a new Narrative Document that is a child of the selected "
      "Narrative Document"),
   (gpointer) newChildNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Narrative Documents"),
   N_("Cut the selected Narrative Documents (you can paste them later)"),
   (gpointer) cutNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Narrative Documents"),
   N_("Delete the selected Narrative Documents"),
   (gpointer) deleteNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Narrative Documents as siblings"),
   N_("Paste preiviously cut Narrative Documents as siblings"),
   (gpointer) pasteSiblingNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Narrative Documents as children"),
   N_("Paste preivously cut Narrative Document as children"),
   (gpointer) pasteChildNarrativeDocument_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Narrative Document properties"),
   N_("Change Narrative Document name, etc"),
   (gpointer) changeNarrativeDocumentProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo frameMenu [] = {
  {GNOME_APP_UI_ITEM,
   N_("New sibling Frame"),
   N_("Create a new Frame that is a sibling of the selected Frame"),
   (gpointer) newSiblingFrame_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("New child Frame"),
   N_("Create a new Frame that is a child of the selected Frame"),
   (gpointer) newChildFrame_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_NEW,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Cut Frames"),
   N_("Cut selected Frames (you can paste them later)"),
   (gpointer) cutFrame_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_CUT,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Delete Frames"),
   N_("Delete selected Frames"),
   (gpointer) deleteFrame_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GTK_STOCK_DELETE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Frames as siblings"),
   N_("Paste preiviously cut Frames as siblings"),
   (gpointer) pasteSiblingFrame_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  {GNOME_APP_UI_ITEM,
   N_("Paste Frames as children"),
   N_("Paste preivously cut Frames as children"),
   (gpointer) pasteChildFrame_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_PIXMAP_PASTE,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_SEPARATOR,

  {GNOME_APP_UI_ITEM,
   N_("Frame properties"),
   N_("Change Frame name, etc"),
   (gpointer) changeFrameProperties_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK,
   GNOME_STOCK_MENU_PROP,
   0, (GdkModifierType) 0, NULL},

  GNOMEUIINFO_END
};

static GnomeUIInfo narrativeAnalysisMenu [] = {
  { GNOME_APP_UI_SUBTREE_STOCK, N_("File"), NULL, narrativeFileMenu,
    NULL, NULL, (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Narrative Document"), NULL,
    narrativeDocumentMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  { GNOME_APP_UI_SUBTREE_STOCK, N_("Frame"), NULL, frameMenu, NULL, NULL,
    (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

  GNOMEUIINFO_MENU_HELP_TREE(helpMenu),
  GNOMEUIINFO_END
};

/** Constructor.

    \param top is the parent window for the GUI

 */
Project::Project(GtkWidget *top)
{
  objType = QualResObject::PROJECT;
  objName = "Project";
  docView = NULL;
  catView = NULL;
  setView = NULL;
  memoView = NULL;
  xrefView = NULL;
  docStore = NULL;
  catStore = NULL;
  setStore = NULL;
  memoStore = NULL;
  xrefStore = NULL;
  narrativeStore = NULL;
  frameStore = NULL;
  searchWindow = NULL;
  searchDestCat = NULL;
  selectSearchCatWindow = NULL;
  memoXrefWindow = NULL;
  mergeCatWindow = NULL;
  mergeCatDestCat = NULL;
  docFilterWindow = NULL;
  selectedSet = NULL;

  narrativeWindow = NULL;

  g_type_init();
  tagTable = gtk_text_tag_table_new();

  createWindow(top);

  // Add the default name.
  setFileName("");
  setSaved(); // Make it think that it is saved

  // Initialize autosave system.
  idleScheduled = FALSE;
  timeoutID = gtk_timeout_add(AUTOSAVE_MSEC,
			      projectTimeout_cb,
			      this);
}

/** Destructor.
 */
Project::~Project()
{
  // Free all Project Components.
  clearComponents();
  if (searchWindow != NULL)
    gtk_widget_destroy(searchWindow);

  if (memoXrefWindow != NULL)
    gtk_widget_destroy(memoXrefWindow);

  if (mergeCatWindow != NULL)
    gtk_widget_destroy(mergeCatWindow);

  if (narrativeWindow != NULL)
    gtk_widget_destroy(narrativeWindow);

  // Destroy the popup menus.
  gtk_widget_destroy(popupDocMenu);
  gtk_widget_destroy(popupCatMenu);
  gtk_widget_destroy(popupSetMenu);
  gtk_widget_destroy(popupMemoMenu);
  gtk_widget_destroy(popupXrefMenu);

  // Remove idle and timeouts.
  if (idleScheduled)
    gtk_idle_remove(idleID);
  gtk_timeout_remove(timeoutID);
}

/** Load the object's disk file name.

    \param fName contains the new disk file name

    This method also updates the autosave file name.

*/
void
Project::setFileName(string fName)
{
  fileName = fName;

  if (fileName == "")
    {
      gtk_window_set_title(GTK_WINDOW(mainWindow), "");
      autoSaveName = "##";
    }
  else
    {
      gchar *dirName, *baseName;

      dirName = g_path_get_dirname(fileName.c_str());
      baseName = g_path_get_basename(fileName.c_str());
      autoSaveName = dirName;
      autoSaveName += G_DIR_SEPARATOR_S;
      autoSaveName += "#";
      autoSaveName += baseName;
      autoSaveName += "#";

      gtk_window_set_title(GTK_WINDOW(mainWindow), baseName);
      g_free(baseName);
      g_free(dirName);
    }
}

/** Create a new component with default name.

\param type contains the type of the new component

\param relationship determines whether to create a sibling or child
*/
void
Project::createComponent(QualResObject::Type type,
			 QualResObject::Relationship relationship)
{
  GtkTreeModel *model;
  GtkTreeIter iter, insertIter, parentIter, *parentIterPtr;
  GtkTreeStore *store;
  GtkWidget *view;
  GtkTreePath *path;
  GtkTreeSelection *selection;

  GtkWidget *errorParentWindow = mainWindow;

  // Container of iterators for selected rows.
  vector<GtkTreeIter> selectedIters;

  ostrstream testName;
  Component *comp;
  gint testNum = 1;

  map<const string, Component *> *compMap;
  string baseName, parentName, compName, compType;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      compName = "0New Document ";
      compType = "Document";
      compMap = (map<const string, Component *> *) &docMap;
      view = docView;
      store = docStore;
      break;
    case QualResObject::CATEGORY:
      compName = "0New Category ";
      compType = "Category";
      compMap = (map<const string, Component *> *) &catMap;
      view = catView;
      store = catStore;
      break;
    case QualResObject::SET:
      compName = "0New Set ";
      compType = "Set";
      compMap = (map<const string, Component *> *) &setMap;
      view = setView;
      store = setStore;
      break;
    case QualResObject::MEMO:
      compName = "0New Memo ";
      compType = "Memo";
      compMap = (map<const string, Component *> *) &memoMap;
      view = memoView;
      store = memoStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::XREF:
      compName = "0New Cross-reference ";
      compType = "Cross-reference";
      compMap = (map<const string, Component *> *) &xrefMap;
      view = xrefView;
      store = xrefStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compName = "0New Narrative Document ";
      compType = "Narrative Document";
      compMap = (map<const string, Component *> *) &narrativeMap;
      view = narrativeView;
      store = narrativeStore;
      errorParentWindow = narrativeWindow;
      break;
    case QualResObject::FRAME:
      compName = "0New Frame ";
      compType = "Frame";
      compMap = (map<const string, Component *> *) &frameMap;
      view = frameView;
      store = frameStore;
      errorParentWindow = narrativeWindow;
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Get the selected Project Components.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  selectedIters.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addItersFromView,
				      (gpointer) &selectedIters);

  // Get the parent iterator, and construct the new base name.
  if (relationship == QualResObject::CHILD)
    {
      // There must be exactly one selected Component.
      if (selectedIters.size() != 1)
	{
	  showMessage("You must first select a single " + compType +
		      " to be the parent of the new " + compType,
		      GTK_MESSAGE_ERROR,
		      errorParentWindow);
	  return;
	}

      // Load the tree iterator for the parent.
      parentIterPtr = &(selectedIters[0]);

      // Load the parent name.
      gtk_tree_model_get(GTK_TREE_MODEL(store),
			 parentIterPtr,
			 QualResObject::ADDRESS, &comp,
			 -1);
      parentName = comp->getName() + SEP_STRING;
    }
  else if (relationship == QualResObject::SIBLING)
    {
      // If there is no selected Component, the Components are pasted
      // at the top level.
      if (selectedIters.size() == 0)
	{
	  parentIterPtr = NULL;
	  parentName = "";
	}
      // If there is a single selected component..
      else if (selectedIters.size() == 1)
	{
	  // Get the parent of this iterator.
	  if (gtk_tree_model_iter_parent(GTK_TREE_MODEL(store),
					 &parentIter,
					 &(selectedIters[0])))
	    {
	      parentIterPtr = &parentIter;

	      // Load the parent name.
	      gtk_tree_model_get(GTK_TREE_MODEL(store),
				 parentIterPtr,
				 QualResObject::ADDRESS, &comp,
				 -1);
	      parentName = comp->getName() + SEP_STRING;
	    }
	  // If there is no parent, this is a top-level paste.
	  else
	    {
	      parentIterPtr = NULL;
	      parentName = "";
	    }
	}
      // Error; there must be zero or one rows selected.
      else
	{
	  showMessage("You must first select no more than one " + compType +
		      " to be the sibling of the new " + compType,
		      GTK_MESSAGE_ERROR,
		      errorParentWindow);
	  return;
	}
    }

  // Create the Component.
  switch (type)
    {
    case QualResObject::DOCUMENT:
      comp = new Document(tagTable);
      break;
    case QualResObject::CATEGORY:
      comp = new Category(tagTable);
      break;
    case QualResObject::SET:
      comp = new Set;
      break;
    case QualResObject::MEMO:
      comp = new Memo(tagTable);
      break;
    case QualResObject::XREF:
      comp = new Xref(tagTable);
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      comp = new NarrativeDocument(tagTable);
      break;
    case QualResObject::FRAME:
      comp = new Frame(tagTable);
      break;
    default:
      g_assert_not_reached();
      return;
    }
  initComponent(comp);

  // Generate the new name.
  baseName = parentName + compName;
  testName << baseName << testNum << ends;
  while (compMap->count(testName.str()) != 0)
    {
      testName.rdbuf()->freeze(0);
      testName.seekp(0, ios::beg);
      testName << baseName << ++testNum << ends;
    }
  
  // Load the name into the component.
  comp->pendingSetName(testName.str());
  comp->commitName();
  testName.rdbuf()->freeze(0);

  // Add the component to the container.
  (*compMap)[comp->getName()] = comp;

  // Get a new iterator.
  gtk_tree_store_append(store, &insertIter, parentIterPtr);

  // Load the values.
  gtk_tree_store_set(store, &insertIter,
		     QualResObject::SHORT_NAME,
		     (comp->getShortName()).c_str(),
		     QualResObject::DESCRIPTION,
		     (comp->getDesc()).c_str(),
		     QualResObject::FULL_NAME,
		     (comp->getName()).c_str(),
		     QualResObject::ADDRESS,
		     comp,
		     -1);

  // Make this row visible.
  if (parentIterPtr != NULL)
    {
      path = gtk_tree_model_get_path(GTK_TREE_MODEL(store),
				     parentIterPtr);
      gtk_tree_view_expand_row(GTK_TREE_VIEW(view),
			       path,
			       FALSE);
      gtk_tree_path_free(path);
    }

  // Indicate that the Project is changed.
  setChanged();
}

/** Update (code, view, etc) selected Project Component.

    \param type contains the type of the component to delete
*/
void
Project::updateComponent(QualResObject::Type type)
{
  GtkWidget *view;
  GtkTreeSelection *selection;
  string compType, compTypes;
  vector<Component *> selectedComps; // Container of Components to open.
  vector<Component*>::iterator iSel;
  GtkWidget *errorParentWindow = mainWindow;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      compType = "Document";
      compTypes = compType + "s";
      view = docView;
      break;
    case QualResObject::CATEGORY:
      compType = "Category";
      compTypes = "Categories";
      view = catView;
      break;
    case QualResObject::SET:
      compType = "Set";
      compTypes = compType + "s";
      view = setView;
      break;
    case QualResObject::MEMO:
      compType = "Memo";
      compTypes = compType + "s";
      view = memoView;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::XREF:
      compType = "Cross-reference";
      compTypes = compType + "s";
      view = xrefView;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compType = "Narrative Document";
      compTypes = compType + "s";
      view = narrativeView;
      break;
    case QualResObject::FRAME:
      compType = "Frame";
      compTypes = compType + "s";
      view = frameView;
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Get the selected Project Component.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  selectedComps.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addAddressesFromView,
				      (gpointer) &selectedComps);

  // Return if there was nothing selected.
  if (selectedComps.size() == 0)
    {
      showMessage("You must first select a " + compType + " or "
		  + compTypes + " to open",
		  GTK_MESSAGE_ERROR,
		  errorParentWindow);
      return;
    }

  // Open the Components.
  for (iSel = selectedComps.begin(); iSel != selectedComps.end(); iSel++)
    (*iSel)->showWindow();
}

/** Cut or Delete the selected Project Components.

    \param type contains the type of the component to remove

    \param cutDel specifies whether the component is to be Cut or
    Deleted
*/
void
Project::removeComponent(QualResObject::Type type, Project::CutDelete cutDel)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreeStore *store;
  GtkWidget *view;
  GtkTreeSelection *selection;
  gchar *name;
  string compType, compTypes, parentComps;
  Component *comp;
  vector<Component *> *cutComps;
  vector<Component *>::iterator iCut;

  GtkWidget *errorParentWindow = mainWindow;

  map<const string, Component *> *compMap;
  string cutDelName;

  // Container of iterators for selected rows.
  vector<GtkTreeIter> selectedIters;
  vector<GtkTreeIter>::iterator iIter;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      compType = "Document";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutDocs;
      compMap = (map<const string, Component *> *) &docMap;
      view = docView;
      store = docStore;
      break;
    case QualResObject::CATEGORY:
      compType = "Category";
      compTypes = "Categories";
      cutComps = (vector<Component *> *) &cutCats;
      compMap = (map<const string, Component *> *) &catMap;
      view = catView;
      store = catStore;
      break;
    case QualResObject::SET:
      compType = "Set";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutSets;
      compMap = (map<const string, Component *> *) &setMap;
      view = setView;
      store = setStore;
      break;
    case QualResObject::MEMO:
      compType = "Memo";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutMemos;
      compMap = (map<const string, Component *> *) &memoMap;
      view = memoView;
      store = memoStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::XREF:
      compType = "Cross-reference";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutXrefs;
      compMap = (map<const string, Component *> *) &xrefMap;
      view = xrefView;
      store = xrefStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compType = "Narrative Document";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutNDocs;
      compMap = (map<const string, Component *> *) &narrativeMap;
      view = narrativeView;
      store = narrativeStore;
      errorParentWindow = narrativeWindow;
      break;
    case QualResObject::FRAME:
      compType = "Frame";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutFrames;
      compMap = (map<const string, Component *> *) &frameMap;
      view = frameView;
      store = frameStore;
      errorParentWindow = narrativeWindow;
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Get the selected Project Components.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  selectedIters.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addItersFromView,
				      (gpointer) &selectedIters);

  // Return if there was nothing selected.
  if (selectedIters.size() == 0)
    {
      showMessage("You must first select a " + compType + " or "
		  + compTypes + " to cut",
		  GTK_MESSAGE_ERROR,
		  errorParentWindow);
      return;
    }

  // Check that each selected Component does not have children.
  parentComps = "";
  for (iIter = selectedIters.begin(); iIter != selectedIters.end(); iIter++)
    {
      gtk_tree_model_get(GTK_TREE_MODEL(store),
			 iIter,
			 QualResObject::ADDRESS, &comp,
			 -1);

      // Check if this Component has children.
      if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), iIter))
	parentComps += "\n\t" + comp->getPrettyName();
    }

  // Display error if a selected Component has children.
  if (parentComps != "")
    {
      showMessage("The following " + compType + " or " + compTypes +
		  " cannot be moved because they have children:\n" +
		  parentComps + "\n\n"
		  "Remove any children if you wish to move the " + compType +
		  " or " + compTypes + ".\n\n"
		  "No " + compTypes + " were moved",
		  GTK_MESSAGE_ERROR,
		  errorParentWindow);
      return;
    }

  // Clear the previously cut Components.
  if (cutDel == CUT)
    {
      for (iCut = cutComps->begin(); iCut != cutComps->end(); iCut++)
	{
	  // Inform all other Project Components of the deletion of
	  // the previously cut component.
	  pendingDelete(*iCut);

	  // Delete the previously cut component.
	  delete *iCut;
	}
    }
  cutComps->clear();

  // Cut the Components.
  for (iIter = selectedIters.begin(); iIter != selectedIters.end(); iIter++)
    {
      gtk_tree_model_get(GTK_TREE_MODEL(store),
			 iIter,
			 QualResObject::ADDRESS, &comp,
			 -1);

      // Inform all Project Components of the cut.
      pendingCut(comp);

      // Cut the Component.
      compMap->erase(comp->getName());
      comp->hideWindow();
      cutComps->push_back(comp);

      // Remove the row.
      gtk_tree_store_remove(store, iIter);
    }

  // Record that the Project has been changed.
  setChanged();
}

/** Paste a Project Component.

    \param type contains the type of the new component

    \param relationship determines whether to paste as a sibling or child
*/
void
Project::pasteComponent(QualResObject::Type type,
			 QualResObject::Relationship relationship)
{
  GtkTreeModel *model;
  GtkTreeIter iter, insertIter, parentIter, *parentIterPtr;

  // Container of iterators for selected rows.
  vector<GtkTreeIter> selectedIters;

  GtkTreeStore *store;
  GtkWidget *view;
  GtkTreePath *path;
  GtkTreeSelection *selection;
  GtkWidget *errorParentWindow = mainWindow;

  ostrstream testName;
  gint testNum = 1;

  map<const string, Component *> *compMap;
  vector<Component *> *cutComps;
  vector<Component *>::iterator iCut;
  Component *comp;

  // Used to generate error messages.
  string compType, compTypes;

  // Used to generate the name of the component.
  string baseName, parentName;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      compType = "Document";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutDocs;
      compMap = (map<const string, Component *> *) &docMap;
      view = docView;
      store = docStore;
      break;
    case QualResObject::CATEGORY:
      compType = "Category";
      compTypes = "Categories";
      cutComps = (vector<Component *> *) &cutCats;
      compMap = (map<const string, Component *> *) &catMap;
      view = catView;
      store = catStore;
      break;
    case QualResObject::SET:
      compType = "Set";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutSets;
      compMap = (map<const string, Component *> *) &setMap;
      view = setView;
      store = setStore;
      break;
    case QualResObject::MEMO:
      compType = "Memo";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutMemos;
      compMap = (map<const string, Component *> *) &memoMap;
      view = memoView;
      store = memoStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::XREF:
      compType = "Cross-reference";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutXrefs;
      compMap = (map<const string, Component *> *) &xrefMap;
      view = xrefView;
      store = xrefStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compType = "Narrative Document";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutNDocs;
      compMap = (map<const string, Component *> *) &narrativeMap;
      view = narrativeView;
      store = narrativeStore;
      errorParentWindow = narrativeWindow;
      break;
    case QualResObject::FRAME:
      compType = "Frame";
      compTypes = compType + "s";
      cutComps = (vector<Component *> *) &cutFrames;
      compMap = (map<const string, Component *> *) &frameMap;
      view = frameView;
      store = frameStore;
      errorParentWindow = narrativeWindow;
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Popup an error window if there is no Project Component that was cut.
  if (cutComps->size() == 0)
    {
      showMessage("You must first cut one or more " + compTypes +
		  " before you can paste them",
		  GTK_MESSAGE_ERROR,
		  errorParentWindow);
      return;
    }

  // Get the selected Project Components.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  selectedIters.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addItersFromView,
				      (gpointer) &selectedIters);

  // Get the parent iterator, and construct the new base name.
  if (relationship == QualResObject::CHILD)
    {
      // There must be exactly one selected Component.
      if (selectedIters.size() != 1)
	{
	  showMessage("You must first select a single " + compType +
		      " to be the parent of the pasted " + compTypes,
		      GTK_MESSAGE_ERROR,
		      errorParentWindow);
	  return;
	}

      // Load the tree iterator for the parent.
      parentIterPtr = &(selectedIters[0]);

      // Load the parent name.
      gtk_tree_model_get(GTK_TREE_MODEL(store),
			 parentIterPtr,
			 QualResObject::ADDRESS, &comp,
			 -1);
      parentName = comp->getName() + SEP_STRING;
    }
  else if (relationship == QualResObject::SIBLING)
    {
      // If there is no selected Component, the Components are pasted
      // at the top level.
      if (selectedIters.size() == 0)
	{
	  parentIterPtr = NULL;
	  parentName = "";
	}
      // If there is a single selected component..
      else if (selectedIters.size() == 1)
	{
	  // Get the parent of this iterator.
	  if (gtk_tree_model_iter_parent(GTK_TREE_MODEL(store),
					 &parentIter,
					 &(selectedIters[0])))
	    {
	      parentIterPtr = &parentIter;

	      // Load the parent name.
	      gtk_tree_model_get(GTK_TREE_MODEL(store),
				 parentIterPtr,
				 QualResObject::ADDRESS, &comp,
				 -1);
	      parentName = comp->getName() + SEP_STRING;
	    }
	  // If there is no parent, this is a top-level paste.
	  else
	    {
	      parentIterPtr = NULL;
	      parentName = "";
	    }
	}
      // Error; there must be zero or one rows selected.
      else
	{
	  showMessage("You must first select no more than one " + compType +
		      " to be the sibling of the pasted " + compTypes,
		      GTK_MESSAGE_ERROR,
		      errorParentWindow);
	  return;
	}
    }

  // Paste the Components.
  for (iCut = cutComps->begin(); iCut != cutComps->end(); iCut++)
    {
      // Create the base name using the name of the parent appended with
      // the short name of the Component to paste.
      baseName = parentName + (*iCut)->getShortName();

      // Generate the new name.
      testName.rdbuf()->freeze(0);
      testName.seekp(0, ios::beg);
      testName << baseName << ends;
      while (compMap->count(testName.str()) != 0)
	{
	  testName.rdbuf()->freeze(0);
	  testName.seekp(0, ios::beg);
	  testName << baseName << " " << testNum++ << ends;
	}

      // Load the new name of the Component to be pasted.
      (*iCut)->pendingSetName(testName.str());
      (*iCut)->commitName();

      // Add the Component to the container.
      (*compMap)[(*iCut)->getName()] = *iCut;

      // Inform all the Components that the Component has been pasted.
      pendingPaste(*iCut);

      // Get a new iterator.
      gtk_tree_store_append(store, &insertIter, parentIterPtr);

      // Load the values.
      gtk_tree_store_set(store, &insertIter,
			 QualResObject::SHORT_NAME,
			 ((*iCut)->getShortName()).c_str(),
			 QualResObject::DESCRIPTION,
			 ((*iCut)->getDesc()).c_str(),
			 QualResObject::FULL_NAME,
			 ((*iCut)->getName()).c_str(),
			 QualResObject::ADDRESS,
			 *iCut,
			 -1);

      // If this is a Category, update the Category tree attributes.
      if ((type == QualResObject::CATEGORY) ||
	  (type == QualResObject::XREF) ||
	  (type == QualResObject::FRAME))
	{
	  Category *cat = (Category *) *iCut;
	  gtk_tree_store_set(store, &insertIter,
			     QualResObject::FG_COLOR, cat->getFgColor(),
			     QualResObject::BG_COLOR, cat->getBgColor(),
			     QualResObject::FONT, (*(cat->getFont())).c_str(),
			     QualResObject::N_DOCS, cat->nDocuments(),
			     -1);
	}

      testName.rdbuf()->freeze(0);

      // Make this row visible.
      if (parentIterPtr != NULL)
	{
	  path = gtk_tree_model_get_path(GTK_TREE_MODEL(store),
					 parentIterPtr);
	  gtk_tree_view_expand_row(GTK_TREE_VIEW(view),
				   path,
				   FALSE);
	  gtk_tree_path_free(path);
	}
    }

  // Clear the cut Components.
  cutComps->clear();

  // Indicate that the Project is changed.
  setChanged();
}

/** Change the properties of the selected Project Component.

    \param type contains the type of the component
*/
void
Project::changeComponentProperties(QualResObject::Type type)
{
  GtkTreeModel *model;
  GtkTreeStore *store;
  GtkWidget *view;
  GtkTreeSelection *selection;
  gchar *name;
  string oldName, newName;
  GtkWidget *errorParentWindow = mainWindow;

  Component *comp; // The component whose properties are changed
  string compName, compDesc, compShortName, compType;
  Category *cat;

  // Container of iterators for selected rows.
  vector<GtkTreeIter> selectedIters;

  map<const string, Component *> *compMap;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      compType = "Document";
      compMap = (map<const string, Component *> *) &docMap;
      view = docView;
      store = docStore;
      break;
    case QualResObject::CATEGORY:
      compType = "Category";
      compMap = (map<const string, Component *> *) &catMap;
      view = catView;
      store = catStore;
      break;
    case QualResObject::SET:
      compType = "Set";
      compMap = (map<const string, Component *> *) &setMap;
      view = setView;
      store = setStore;
      break;
    case QualResObject::MEMO:
      compType = "Memo";
      compMap = (map<const string, Component *> *) &memoMap;
      view = memoView;
      store = memoStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::XREF:
      compType = "Cross-reference";
      compMap = (map<const string, Component *> *) &xrefMap;
      view = xrefView;
      store = xrefStore;
      errorParentWindow = memoXrefWindow;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compType = "Narrative Document";
      compMap = (map<const string, Component *> *) &narrativeMap;
      view = narrativeView;
      store = narrativeStore;
      errorParentWindow = narrativeWindow;
      break;
    case QualResObject::FRAME:
      compType = "Frame";
      compMap = (map<const string, Component *> *) &frameMap;
      view = frameView;
      store = frameStore;
      errorParentWindow = narrativeWindow;
      break;
    default:
      g_assert_not_reached();
      return;
    }
  // Get the selected Project Component.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  selectedIters.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addItersFromView,
				      (gpointer) &selectedIters);

  // Popup error box if there's no selection.
  if (selectedIters.size() != 1)
    {
      showMessage("Please select a single " + compType +
		  " whose properties you wish to change",
		  GTK_MESSAGE_ERROR,
		  errorParentWindow);
      return;
    }

  gtk_tree_model_get(GTK_TREE_MODEL(store),
		     &(selectedIters[0]),
		     QualResObject::ADDRESS, &comp,
		     -1);
  // Save original name.
  oldName = comp->getName();

  // Change properties.
  if (comp->changeProperties(GTK_WINDOW(mainWindow)) == false)
    return; // Aborted

  newName = comp->getName();

  // Check whether the name of the component is unique.
  if ((compMap->count(newName) != 0) &&
      (oldName != newName))
    {
      // Popup window.
      showMessage("There is already a " + compType +
		  " with the name " + comp->getName() +
		  "\n\nThe name was not changed",
		  GTK_MESSAGE_ERROR,
		  errorParentWindow);

      // Restore old name.
      comp->uncommitName();
    }
  else
    {
      // Commit the new name, and mark the Project as changed.
      comp->commitName();
      setChanged();

      // Update the map.
      compMap->erase(oldName);
      (*compMap)[newName] = comp;

      // Update the name of this Project Component's children, and
      // inform all other Project Components of the name change.
      updateChildrenName(compMap,
			 store,
			 &(selectedIters[0]),
			 oldName,
			 newName);
    }

  // Update the tree.
  compName = comp->getName();
  compShortName = comp->getShortName();
  compDesc = comp->getDesc();
  gtk_tree_store_set(store, &(selectedIters[0]),
		     QualResObject::SHORT_NAME, compShortName.c_str(),
		     QualResObject::DESCRIPTION, compDesc.c_str(),
		     QualResObject::FULL_NAME, compName.c_str(),
		     QualResObject::ADDRESS, comp,
		     -1);

  // If this is a Category, update the Category tree attributes.
  if ((type == QualResObject::CATEGORY) ||
      (type == QualResObject::XREF) ||
      (type == QualResObject::FRAME))
    {
      cat = (Category *) comp;
      gtk_tree_store_set(store, &(selectedIters[0]),
			 QualResObject::FG_COLOR, cat->getFgColor(),
			 QualResObject::BG_COLOR, cat->getBgColor(),
			 QualResObject::FONT, (*(cat->getFont())).c_str(),
			 -1);
    }

  // Inform all Project Components.
  propertiesHaveChanged(comp);
}

/** Print the names and descriptions of the Components

    \param type contains the type of the component
*/
void
Project::printComponentList(QualResObject::Type type)
{
  PrintObject print;
  string buffer;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      print.setLabeledName("Documents for " + fileName);
      break;
    case QualResObject::CATEGORY:
      print.setLabeledName("Categories for " + fileName);
      break;
    case QualResObject::MEMO:
      print.setLabeledName("Memos for " + fileName);
      break;
    case QualResObject::XREF:
      print.setLabeledName("Cross-references for " + fileName);
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      print.setLabeledName("Narrative Documents for " + fileName);
      break;
    case QualResObject::FRAME:
      print.setLabeledName("Frames for " + fileName);
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Set the parent window of the print object.
  print.setParentWindow(&mainWindow);

  // Load the list of Components.
  buffer = getComponentList(type);
  print.loadTextToPrint(buffer);

  // Print.
  print.print();
}

/** Print all the Components

    \param type contains the type of the component
*/
void
Project::printAllComponents(QualResObject::Type type)
{
  map<const string, Component *> *compMap;
  map<const string, Component *>::iterator iComp;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      compMap = (map<const string, Component *> *) &docMap;
      break;
    case QualResObject::CATEGORY:
      compMap = (map<const string, Component *> *) &catMap;
      break;
    case QualResObject::MEMO:
      compMap = (map<const string, Component *> *) &memoMap;
      break;
    case QualResObject::XREF:
      compMap = (map<const string, Component *> *) &xrefMap;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compMap = (map<const string, Component *> *) &narrativeMap;
      break;
    case QualResObject::FRAME:
      compMap = (map<const string, Component *> *) &frameMap;
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Print each component.
  for (iComp = compMap->begin(); iComp != compMap->end(); iComp++)
    {
      (iComp->second)->print();
    }
}

/** Export the names and descriptions of the Components

    \param type contains the type of the component
*/
void
Project::exportComponentList(QualResObject::Type type)
{
  string buffer;
  GtkWidget *fileSelect;
  string exportFileName;
  ofstream file;
  gint result;

  // Pop up file selection box.
   fileSelect = gtk_file_selection_new("Please select a file for export");
   
   // Display.
   gtk_widget_show(fileSelect);
   result = gtk_dialog_run(GTK_DIALOG(fileSelect));
   exportFileName = gtk_file_selection_get_filename
     (GTK_FILE_SELECTION(fileSelect));
   gtk_widget_destroy(GTK_WIDGET(fileSelect));

  // Abort.
   if ((result != GTK_RESPONSE_OK) ||
       g_file_test(exportFileName.c_str(), G_FILE_TEST_IS_DIR))
    return;

  // Load the list of Components.
  buffer = getComponentList(type);

  // Export.
  file.open(exportFileName.c_str(), ios::out | ios::trunc);
  if (file.bad())
    {
      showMessage("Error opening file '" + fileName + "'",
		  GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  switch (type)
    {
    case QualResObject::DOCUMENT:
      file << "Documents for " + fileName << "--\n\n";
      break;
    case QualResObject::CATEGORY:
      file << "Categories for " + fileName << "--\n\n";
      break;
    case QualResObject::MEMO:
      file << "Memos for " + fileName << "--\n\n";
      break;
    case QualResObject::XREF:
      file << "Cross-references for " + fileName << "--\n\n";
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      file << "Narrative Documents for " + fileName << "--\n\n";
      break;
    case QualResObject::FRAME:
      file << "Frames for " + fileName << "--\n\n";
      break;
    default:
      g_assert_not_reached();
      return;
    }

  file << buffer;
  file.close();
}

/** Load the list of Components.

    \param type contains the type of the new component

    \return formatted list of Components
*/
string
Project::getComponentList(QualResObject::Type type)
{
  string buffer, descBuffer;
  gint indx;

  // Component Project store.
  GtkTreeStore *store;

  // Address of Components.
  vector<Component *> comp;
  vector<Component *>::iterator iComp;

  switch (type)
    {
    case QualResObject::DOCUMENT:
      store = docStore;
      break;
    case QualResObject::CATEGORY:
      store = catStore;
      break;
    case QualResObject::MEMO:
      store = memoStore;
      break;
    case QualResObject::XREF:
      store = xrefStore;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      store = narrativeStore;
      break;
    case QualResObject::FRAME:
      store = frameStore;
      break;
    default:
      g_assert_not_reached();
      return(buffer);
    }

  // Get the names of each Component.
  comp.clear();
  gtk_tree_model_foreach(GTK_TREE_MODEL(store),
			 loadComponentAddressesFromStore,
			 (gpointer) &comp);

  // Load names and descriptions.
  for (iComp = comp.begin(); iComp != comp.end(); iComp++)
    {
      buffer += (*iComp)->getPrettyName() + "\n";
      descBuffer = (*iComp)->getDesc();

      // Add padding before description.
      indx = 0;
      while ((indx = descBuffer.find(NEWLINE_CHAR, indx)) != string::npos)
	descBuffer.insert(++indx, PRINT_DESCRIPTION_PAD);

      if (descBuffer != "")
	buffer += PRINT_DESCRIPTION_PAD + descBuffer + "\n\n";
      else
	buffer += "\n";
    }

  return(buffer);
}

/** Update the name of a Project Components children.

    \param compMap is the Project Component map

    \param store is the Project tree model

    \param parentIter points to the Project Component in the tree view

    \param oldName is the old name of the Project Component

    \param newName is the new name of the Project Component

    The name of the Project Component is newName.  The names of all
    its children are changed.  All Project Components are informed of
    the name change.
*/
void
Project::updateChildrenName(map<const string, Component *> *compMap,
			    GtkTreeStore *store,
			    GtkTreeIter *parentIter,
			    string &oldName,
			    string &newName)
{
  gint oldNameLength = oldName.length();
  Component *comp;
  string compName;
  GtkTreeIter childIter;
  gint nChildren;
  gchar *name;
  gint i;

  if (newName == oldName)
    return;

  nChildren = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store),
					     parentIter);
// For each child...
  for (i = 0; i < nChildren; i++)
    {
      // Get the iterator for the child.
      gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store),
				    &childIter,
				    parentIter,
				    i);

      // Get the child's name.
      gtk_tree_model_get(GTK_TREE_MODEL(store),
			 &childIter,
			 QualResObject::FULL_NAME, &name, -1);

      // Rename the child.
      comp = (*compMap)[name];
      compName = comp->getName();
      compName = newName + compName.substr(oldNameLength);
      comp->pendingSetName(compName);
      comp->commitName();
      compMap->erase(name);
      (*compMap)[compName] = comp;

      // Update the tree.
      gtk_tree_store_set(store, &childIter,
			 QualResObject::SHORT_NAME,
			 (comp->getShortName()).c_str(),
			 QualResObject::FULL_NAME, compName.c_str(),
			 QualResObject::ADDRESS, comp,
			 -1);

      // Inform all Project Components.
      propertiesHaveChanged(comp);

      // Update the names of the children.
      updateChildrenName(compMap, store, &childIter, oldName, newName);

      g_free(name);
    }

  if (nChildren != 0)
    setChanged();
}

/** Free resources for all Project Components
*/
void
Project::clearComponents()
{
  map<const string , Document *>::iterator docPtr;
  map<const string , Category *>::iterator catPtr;
  map<const string , Set *>::iterator setPtr;
  map<const string , Memo *>::iterator memoPtr;
  map<const string , Xref *>::iterator xrefPtr;
  map<const string , NarrativeDocument *>::iterator narrativePtr;
  map<const string , Frame *>::iterator framePtr;

  vector<Document *>::iterator iCutDocs;
  vector<Category *>::iterator iCutCats;
  vector<Set *>::iterator iCutSets;
  vector<Memo *>::iterator iCutMemos;
  vector<Xref *>::iterator iCutXrefs;
  vector<NarrativeDocument *>::iterator iCutNDocs;
  vector<Frame *>::iterator iCutFrames;

  // Free memory allocated to Project Components.
  for (docPtr = docMap.begin(); docPtr != docMap.end(); docPtr++)
      delete docMap[docPtr->first];
  docMap.clear();

  for (catPtr = catMap.begin(); catPtr != catMap.end(); catPtr++)
    delete catMap[catPtr->first];
  catMap.clear();

  for (setPtr = setMap.begin(); setPtr != setMap.end(); setPtr++)
    delete setMap[setPtr->first];
  setMap.clear();

  for (memoPtr = memoMap.begin(); memoPtr != memoMap.end(); memoPtr++)
    delete memoMap[memoPtr->first];
  memoMap.clear();

  for (xrefPtr = xrefMap.begin(); xrefPtr != xrefMap.end(); xrefPtr++)
    delete xrefMap[xrefPtr->first];
  xrefMap.clear();

  for (narrativePtr = narrativeMap.begin();
       narrativePtr != narrativeMap.end();
       narrativePtr++)
    delete narrativeMap[narrativePtr->first];
  narrativeMap.clear();

  for (framePtr = frameMap.begin(); framePtr != frameMap.end(); framePtr++)
    delete frameMap[framePtr->first];
  frameMap.clear();

  // Free memory allocated to cut Project Components.
  for (iCutDocs = cutDocs.begin(); iCutDocs != cutDocs.end(); iCutDocs++)
      delete *iCutDocs;
  cutDocs.clear();

  for (iCutCats = cutCats.begin(); iCutCats != cutCats.end(); iCutCats++)
      delete *iCutCats;
  cutCats.clear();

  for (iCutSets = cutSets.begin(); iCutSets != cutSets.end(); iCutSets++)
      delete *iCutSets;
  cutSets.clear();

  for (iCutMemos = cutMemos.begin(); iCutMemos != cutMemos.end(); iCutMemos++)
      delete *iCutMemos;
  cutMemos.clear();

  for (iCutXrefs = cutXrefs.begin(); iCutXrefs != cutXrefs.end(); iCutXrefs++)
      delete *iCutXrefs;
  cutXrefs.clear();

  for (iCutNDocs = cutNDocs.begin(); iCutNDocs != cutNDocs.end(); iCutNDocs++)
      delete *iCutNDocs;
  cutNDocs.clear();
}

/** Has the Project been saved?

    \return true if the Project and all Components have been saved to
    the Project file.
*/
bool
Project::projectSaved()
{
  bool projSaved;

  map<const string, Document *>::iterator iDoc;
  map<const string, Category *>::iterator iCat;
  map<const string, Set *>::iterator iSet;
  map<const string, Memo *>::iterator iMemo;
  map<const string, Xref *>::iterator iXref;
  map<const string, NarrativeDocument *>::iterator iNDoc;
  map<const string, Frame *>::iterator iFrame;

  projSaved = saved();

  for (iDoc = docMap.begin(); iDoc != docMap.end(); iDoc++)
    projSaved = projSaved && (iDoc->second)->saved();

  for (iCat = catMap.begin(); iCat != catMap.end(); iCat++)
    projSaved = projSaved && (iCat->second)->saved();

  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
    projSaved = projSaved && (iSet->second)->saved();

  for (iMemo = memoMap.begin(); iMemo != memoMap.end(); iMemo++)
    projSaved = projSaved && (iMemo->second)->saved();

  for (iXref = xrefMap.begin(); iXref != xrefMap.end(); iXref++)
    projSaved = projSaved && (iXref->second)->saved();

  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    projSaved = projSaved && (iNDoc->second)->saved();

  for (iFrame = frameMap.begin(); iFrame != frameMap.end(); iFrame++)
    projSaved = projSaved && (iFrame->second)->saved();

  return(projSaved);
}

/** Save the Project to file (obtaining file name if necessary).

    \return indication whether the Project was saved

    The Project name is the name of the file to save the Project to.

    If the Project name is "", then a file selection window is popped
    up.  After the Project name is loaded (assuming "" was not
    entered), the Project is saved.

*/
enum Project::SaveDiscard
Project::saveToFile()
{
  GtkWidget *dialog; // Status popup
  GtkWidget *fileSelect; // File selection popup
  gint result;

  // Obtain a file name if none has been specified.
  if (fileName == "")
    {
      // Pop up file selection box.
      fileSelect = gtk_file_selection_new("Save Project as");
      gtk_file_selection_complete(GTK_FILE_SELECTION(fileSelect),
				  PROJECT_FILE_PATTERN);

      // Display.
      gtk_widget_show(fileSelect);
      result = gtk_dialog_run(GTK_DIALOG(fileSelect));
      fileName =
	gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileSelect));
      setFileName(fileName);
      gtk_widget_destroy(GTK_WIDGET(fileSelect));

      // Abort.
      if ((result != GTK_RESPONSE_OK) ||
	  g_file_test(fileName.c_str(), G_FILE_TEST_IS_DIR))
	{
	  // Pop up message box.
	  dialog = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
					  GTK_DIALOG_DESTROY_WITH_PARENT,
					  GTK_MESSAGE_INFO,
					  GTK_BUTTONS_CLOSE,
					  "Project save aborted");
	  gtk_dialog_run(GTK_DIALOG(dialog));
	  gtk_widget_destroy(dialog);

	  // Reset and abort;
	  fileName = "";
	  setFileName(fileName);
	  return(Project::CANCEL);
	}
    }

  // Save the file.
  save();
  return(Project::SAVE);
}

/** Save the Project to a new file.

*/
void
Project::saveAsToFile()
{
  GtkWidget *dialog; // Status popup
  GtkWidget *fileSelect; // File selection popup
  gint result;
  string fName;

  // Pop up file selection box.
  fileSelect = gtk_file_selection_new("Save Project as");
  gtk_file_selection_complete(GTK_FILE_SELECTION(fileSelect),
			      PROJECT_FILE_PATTERN);
   
  // Display.
  gtk_widget_show(fileSelect);
  result = gtk_dialog_run(GTK_DIALOG(fileSelect));
  fName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileSelect));
  gtk_widget_destroy(GTK_WIDGET(fileSelect));

  // Abort.
  if ((result != GTK_RESPONSE_OK) ||
      g_file_test(fName.c_str(), G_FILE_TEST_IS_DIR))
    {
      // Pop up message box.
      dialog = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
				      GTK_DIALOG_DESTROY_WITH_PARENT,
				      GTK_MESSAGE_INFO,
				      GTK_BUTTONS_CLOSE,
				      "Project save aborted");
      gtk_dialog_run(GTK_DIALOG(dialog));
      gtk_widget_destroy(dialog);

      return;
    }

  // Save the file.
  setFileName(fName);
  save();
}

/** Close the Project (after checking that the Project has been saved).

    If the Project has not been saved, the user is given the option of
    saving or discarding the Project, or canceling the close
    operation.

    \return the action

*/
enum Project::SaveDiscard
Project::closeProject()
{
  GtkWidget *dialog; // Status popup
  enum Project::SaveDiscard action;
  gint result;

  action = Project::SAVE;

  // Give the user the option to save the Project.
  if (!projectSaved())
    {
      // Pop up the warning.
      dialog = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
				      GTK_DIALOG_DESTROY_WITH_PARENT,
				      GTK_MESSAGE_QUESTION,
				      GTK_BUTTONS_NONE,
				      "Save the Project before "
				      "closing it?");
      gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			     GTK_STOCK_SAVE, Project::SAVE,
			     GTK_STOCK_NO, Project::DISCARD,
			     GTK_STOCK_CANCEL, Project::CANCEL,
			     NULL);

      result = gtk_dialog_run(GTK_DIALOG(dialog));
      gtk_widget_destroy(dialog);
   
      switch (result)
	{
	case Project::CANCEL:
	  action = Project::CANCEL;
	  break;
	case Project::SAVE:
	  action = saveToFile();
	  break;
	case Project::DISCARD:
	  action = Project::DISCARD;
	  break;
	default:
	  g_assert_not_reached();
	  return(Project::CANCEL);
	}
    }

  // Close the Project.
  if (action != Project::CANCEL)
    close();

  return(action);
}

/** Close the Project

    All the Components are freed, and the autosave file is unlinked.
*/
void
Project::close()
{
  string emptyString = "";

  // Free all Project Components.
  clearComponents();

  // Update the displays.
  updateTreeStore(QualResObject::DOCUMENT);
  updateTreeStore(QualResObject::CATEGORY);
  updateTreeStore(QualResObject::SET);
  updateTreeStore(QualResObject::MEMO);
  updateTreeStore(QualResObject::XREF);
  updateTreeStore(QualResObject::NARRATIVE_DOCUMENT);
  updateTreeStore(QualResObject::FRAME);

  // Remove any autosave file.
  unlink(autoSaveName.c_str());

  // Clear the name, description, and file name.
  setFileName("");
  setDesc(emptyString);
  setSaved();
}

/** Save the Project to file.

    The name of the file to save to is the Project name.  The Project
    name must be the name of a valid file.  If there is an error, a
    status window is popped up.

    This method makes a backup of the original file, if there is one.
    The name of the backup is obtained by adding BACKUP_SUFFIX to the
    file name.
*/
void
Project::save()
{
  GdomeDOMImplementation *domimpl;
  GdomeDocument *xmlDoc;
  GdomeElement *root;
  GdomeException exc;
  GdomeDOMString *tempStr, *tempVal;
  string errorMessage = "Error saving to file'" + fileName + "'";

  // Get a DOMImplementation reference.
  domimpl = gdome_di_mkref();

  // Create a new document with Project as root element.
  tempStr = gdome_str_mkref(PROJECT_TAG);
  xmlDoc = gdome_di_createDocument(domimpl, NULL, tempStr, NULL, &exc);
  gdome_str_unref(tempStr);
  if (xmlDoc == NULL)
    {
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Get reference to the root element of the document.
  root = gdome_doc_documentElement(xmlDoc, &exc);
  if (root == NULL)
    {
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  try
    {
      // Save everything to the root element.
      saveToRoot(xmlDoc, root);
    }
  catch(FileException)
    {
      gdome_el_unref(root, &exc);
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  gdome_el_unref(root, &exc);

  // Save existing Project file, if any.
  if (!g_file_test(fileName.c_str(), G_FILE_TEST_IS_DIR) &&
       g_file_test(fileName.c_str(), G_FILE_TEST_EXISTS))
    {
      // Construct backup name.
      string backupName = fileName + BACKUP_SUFFIX;

      // Rename original file.
      if (rename(fileName.c_str(), backupName.c_str()))
	{
	  string backupMessage = "Error creating backup file:  ";
	  backupMessage += sys_errlist[errno];
	  showMessage(backupMessage, GTK_MESSAGE_ERROR, mainWindow);
	}
    }

  // Save the Project to file.
  if (!gdome_di_saveDocToFile(domimpl, xmlDoc, fileName.c_str(),
			      GDOME_SAVE_STANDARD, &exc))
    {
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Remove any autosave file.
  unlink(autoSaveName.c_str());

  // Free the document structure and the DOMImplementation.
  gdome_di_freeDoc(domimpl, xmlDoc, &exc);
  gdome_di_unref(domimpl, &exc);

  setSaved();
}

/** Autosave the Project to file.

    The name of the autosave file is obtained from the Project file
    name.  The autosave file name must be the name of a valid file.
    If there is an error, a status window is popped up.

*/
void
Project::autoSave()
{
  GdomeDOMImplementation *domimpl;
  GdomeDocument *xmlDoc;
  GdomeElement *root;
  GdomeException exc;
  GdomeDOMString *tempStr, *tempVal;
  string errorMessage, backupName = "";

  errorMessage = "Error autosaving file '" + fileName + "'\n\n"
    "Autosave file:  '" + autoSaveName + "'";

  // Get a DOMImplementation reference.
  domimpl = gdome_di_mkref();

  // Create a new document with Project as root element.
  tempStr = gdome_str_mkref(PROJECT_TAG);
  xmlDoc = gdome_di_createDocument(domimpl, NULL, tempStr, NULL, &exc);
  gdome_str_unref(tempStr);
  if (xmlDoc == NULL)
    {
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Get reference to the root element of the document.
  root = gdome_doc_documentElement(xmlDoc, &exc);
  if (root == NULL)
    {
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  try
    {
      // Save everything to the root element.
      saveToRoot(xmlDoc, root);
    }
  catch(FileException)
    {
      gdome_el_unref(root, &exc);
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  gdome_el_unref(root, &exc);

  // Save existing autosave file, if any.
  if (!g_file_test(autoSaveName.c_str(), G_FILE_TEST_IS_DIR) &&
       g_file_test(autoSaveName.c_str(), G_FILE_TEST_EXISTS))
    {
      // Construct backup name.
      backupName = autoSaveName + BACKUP_SUFFIX;

      // Rename original file.
      if (rename(autoSaveName.c_str(), backupName.c_str()))
	{
	  string backupMessage = "Error creating backup autosave file:  ";
	  backupMessage += sys_errlist[errno];
	  showMessage(backupMessage, GTK_MESSAGE_ERROR, mainWindow);
	}
    }

  // Autosave the Project.
  if (!gdome_di_saveDocToFile(domimpl, xmlDoc, autoSaveName.c_str(),
			      GDOME_SAVE_STANDARD, &exc))
    {
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);

      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Remove any backup autosave file.
  if (backupName != "")
    unlink(backupName.c_str());

  // Free the document structure and the DOMImplementation.
  gdome_di_freeDoc(domimpl, xmlDoc, &exc);
  gdome_di_unref(domimpl, &exc);
}

/** Save all the Project to the root element.

    \param xmlDoc is the pointer to the XML document.

    \param root is the XML root element

*/
void
Project::saveToRoot(GdomeDocument *xmlDoc, GdomeElement *root)
{
  GdomeElement *el;
  GdomeException exc;
  GdomeNode *result;

  map<const string, Document *>::iterator iDoc;
  map<const string, Category *>::iterator iCat;
  map<const string, Set *>::iterator iSet;
  map<const string, Memo *>::iterator iMemo;
  map<const string, Xref *>::iterator iXref;
  map<const string, NarrativeDocument *>::iterator iNDoc;
  map<const string, Frame *>::iterator iFrame;

  // Set the PACKAGE attribute.
  setAttribute(root, PACKAGE_ATTR, PACKAGE_NAME);

  // Set the VERSION attribute.
  setAttribute(root, VERSION_ATTR, PACKAGE_VERSION);

  // Create and write the description element.
  setTextInElement(xmlDoc, root,
		   DESCRIPTION_TAG,
		   (gchar *) (getDesc()).c_str());

  // Save the Documents.
  for (iDoc = docMap.begin(); iDoc != docMap.end(); iDoc++)
    {
      // Create a Document element.
      el = (iDoc->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild(root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }

  // Save the Categories.
  for (iCat = catMap.begin(); iCat != catMap.end(); iCat++)
    {
      // Create a Category element.
      el = (iCat->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild (root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }

  // Save the Sets.
  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
    {
      // Create a Set element.
      el = (iSet->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild (root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }

  // Save the Memos.
  for (iMemo = memoMap.begin(); iMemo != memoMap.end(); iMemo++)
    {
      // Create a Memo element.
      el = (iMemo->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild(root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }

  // Save the Cross-references.
  for (iXref = xrefMap.begin(); iXref != xrefMap.end(); iXref++)
    {
      // Create a Cross-reference element.
      el = (iXref->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild(root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }

  // Save the Narrative Documents.
  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    {
      // Create a Narrative Document element.
      el = (iNDoc->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild(root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }

  // Save the Frames.
  for (iFrame = frameMap.begin(); iFrame != frameMap.end(); iFrame++)
    {
      // Create a Frame element.
      el = (iFrame->second)->writeXML(xmlDoc);

      // Append the element to the childs list of the root element.
      result = gdome_el_appendChild(root, (GdomeNode *)el, &exc);
      if (result != (GdomeNode *)el) {
	fprintf (stderr, "Element.appendChild: failed\n"
		 "\tException #%d\n", exc);
      }
      gdome_el_unref(el, &exc);
    }
}

/** Restore a Project from a file, obtaining file name.

This method first checks that there is no currently-open Project.

*/
void
Project::restoreFromFile()
{
  GtkWidget *dialog; // Status popup
  GtkWidget *fileSelect; // File selection popup
  string tempFileName;
  gint result;

  // Obtain a file name.
  // Pop up file selection box.
  fileSelect = gtk_file_selection_new("Open Project");
  gtk_file_selection_complete(GTK_FILE_SELECTION(fileSelect),
			      PROJECT_FILE_PATTERN);
   
  // Display and get command.
  gtk_widget_show(fileSelect);
  result = gtk_dialog_run(GTK_DIALOG(fileSelect));

  tempFileName =  gtk_file_selection_get_filename
    (GTK_FILE_SELECTION(fileSelect));
  gtk_widget_destroy(GTK_WIDGET(fileSelect));

  // Abort.
  if ((result != GTK_RESPONSE_OK) ||
      g_file_test(fileName.c_str(), G_FILE_TEST_IS_DIR))
    {
      // Pop up message box.
      showMessage("Project open aborted",
		  GTK_MESSAGE_INFO,
		  mainWindow);
      return;
    }

  // Close any currently-open Project.
  if (closeProject() == Project::CANCEL)
    return;

  // Restore the Project.
  setFileName(tempFileName);
  restore();
}

/** Restore the Project from file specified by the Project name..

    This assumes that the Project name is the name of a valid file.
    If there is an error, a status window is popped up.

*/
void
Project::restore()
{
  GdomeDOMImplementation *domimpl;
  GdomeDocument *xmlDoc;
  GdomeElement *root;
  GdomeNodeList *compElems;
  GdomeException exc;
  enum AutoRecover autoRecover;
  string fName;
  string errorMessage = "Error reading file'" + fileName + "'";
  map<const string, NarrativeDocument *>::iterator iNDoc;

  // Determine whether to autorecover.
  autoRecover = autoRecoverFile(fileName, autoSaveName, mainWindow);
  if (autoRecover == Project::READ_AUTOSAVE)
    fName = autoSaveName;
  else if (autoRecover == Project::READ_PROJECT)
    fName = fileName;
  else if (autoRecover == Project::CANCEL_READ)
    {
      setFileName("");
      return;
    }
  else
    {
      setFileName("");
      g_return_if_reached();
    }

  // Get a DOMImplementation reference.
  domimpl = gdome_di_mkref();

  // Load the new document.
  xmlDoc = gdome_di_createDocFromURI(domimpl, fName.c_str(),
				      GDOME_LOAD_PARSING, &exc);
  if (xmlDoc == NULL) {
    showMessage("Error opening file '" + fName + "'",
		GTK_MESSAGE_ERROR,
		mainWindow);
    if (!autoRecover)
      setFileName("");
    gdome_di_unref(domimpl, &exc);
    return;
  }

  // Get reference to the root element of the document.
  root = gdome_doc_documentElement(xmlDoc, &exc);
  if (root == NULL) {
    gdome_di_freeDoc(domimpl, xmlDoc, &exc);
    gdome_di_unref(domimpl, &exc);
    showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
    return;
  }

  try
    {
      // Load the description.
      setDesc(getTextFromElement(root, DESCRIPTION_TAG));
    }
  catch(FileException)
    {
      gdome_el_unref(root, &exc);
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);
      closeProject();
      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Get the reference to all the Project Components.
  compElems = gdome_el_childNodes(root, &exc);
  if (compElems == NULL)
    {
      gdome_el_unref(root, &exc);
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);
      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Restore all Components.
  try
    {
      restoreComponents(compElems, QualResObject::DOCUMENT);
      restoreComponents(compElems, QualResObject::CATEGORY);
      restoreComponents(compElems, QualResObject::SET);
      restoreComponents(compElems, QualResObject::MEMO);
      restoreComponents(compElems, QualResObject::XREF);
      restoreComponents(compElems, QualResObject::NARRATIVE_DOCUMENT);
      restoreComponents(compElems, QualResObject::FRAME);
    }
  catch(FileException)
    {
      gdome_nl_unref(compElems, &exc);
      gdome_el_unref(root, &exc);
      gdome_di_freeDoc(domimpl, xmlDoc, &exc);
      gdome_di_unref(domimpl, &exc);
      closeProject();
      showMessage(errorMessage, GTK_MESSAGE_ERROR, mainWindow);
      return;
    }

  // Free XML structures.
  gdome_nl_unref(compElems, &exc);
  gdome_el_unref(root, &exc);
  gdome_di_freeDoc(domimpl, xmlDoc, &exc);
  gdome_di_unref(domimpl, &exc);

  // Display the Project Components.
  updateTreeStore(QualResObject::DOCUMENT);
  updateTreeStore(QualResObject::CATEGORY);
  updateTreeStore(QualResObject::SET);
  updateTreeStore(QualResObject::MEMO);
  updateTreeStore(QualResObject::XREF);
  updateTreeStore(QualResObject::NARRATIVE_DOCUMENT);
  updateTreeStore(QualResObject::FRAME);

  // Restoring the Frames changes the Narrative Documents.  Make the
  // saved flag true.
  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    (iNDoc->second)->setSaved();

  // If we read from the Project file, assume the Project is saved.
  if (autoRecover == Project::READ_PROJECT)
    setSaved();
  else
    setChanged();
}

/** Load all Project Components of a given type.

    \param elems is the list of child elements of the Project XML
    element

    \param type contains the type of the new component

*/
void
Project::restoreComponents(GdomeNodeList *elems, QualResObject::Type type)
  throw(FileException)
{
  GdomeElement *el;
  GdomeException exc;
  GdomeDOMString *tempStr;
  gint i, nComponents;
  string compType;
  Document *doc;
  Category *cat;
  Set *set;
  Memo *memo;
  Xref *xref;
  NarrativeDocument *ndoc;
  Frame *frame;
  map<const string, Set *>::iterator iSet;

  nComponents = gdome_nl_length(elems, &exc);

  // Load each Project Component.
  for (i = 0; i < nComponents; i++)
    {
      // Obtain the next Project Component.
      el = (GdomeElement *)gdome_nl_item(elems, i, &exc);
      if (el == NULL)
	throw FileException();

      // Check that it is an element.
      if (gdome_el_nodeType(el, &exc) == GDOME_ELEMENT_NODE)
	{
	  // It is.  Get the tag name.
	  tempStr = gdome_el_tagName(el, &exc);
	  if (exc)
	    {
	      gdome_el_unref (el, &exc);
	      throw FileException();
	    }
	  compType = tempStr->str;
	  gdome_str_unref(tempStr);

	  // Create the appropriate Project Component.
	  try
	    {
	      switch (type)
		{
		case QualResObject::DOCUMENT:
		  if (compType == DOCUMENT_TAG)
		    {
		      // Create new Document.
		      doc = new Document(tagTable);
		      initComponent(doc);
		      doc->readXML(el);
		      docMap[doc->getName()] = doc;
		    }
		  break;
		case QualResObject::CATEGORY:
		  if (compType == CATEGORY_TAG)
		    {
		      cat = new Category(tagTable);
		      initComponent(cat);
		      cat->readXML(el);
		      catMap[cat->getName()] = cat;
		    }
		  break;
		case QualResObject::SET:
		  if (compType == SET_TAG)
		    {
		      set = new Set;
		      initComponent(set);
		      set->readXML(el);
		      setMap[set->getName()] = set;
		    }
		  break;
		case QualResObject::MEMO:
		  if (compType == MEMO_TAG)
		    {
		      // Create new Memo.
		      memo = new Memo(tagTable);
		      initComponent(memo);
		      memo->readXML(el);
		      memoMap[memo->getName()] = memo;
		    }
		  break;
		case QualResObject::XREF:
		  if (compType == XREF_TAG)
		    {
		      // Create new Cross-reference
		      xref = new Xref(tagTable);
		      initComponent(xref);
		      xref->readXML(el);
		      xrefMap[xref->getName()] = xref;
		    }
		  break;
		case QualResObject::NARRATIVE_DOCUMENT:
		  if (compType == NARRATIVE_DOCUMENT_TAG)
		    {
		      // Create new Narrative Document.
		      ndoc = new NarrativeDocument(tagTable);
		      initComponent(ndoc);
		      ndoc->readXML(el);
		      narrativeMap[ndoc->getName()] = ndoc;
		    }
		  break;
		case QualResObject::FRAME:
		  if (compType == FRAME_TAG)
		    {
		      frame = new Frame(tagTable);
		      initComponent(frame);
		      frame->readXML(el);
		      frameMap[frame->getName()] = frame;
		    }
		  break;
		default:
		  g_assert_not_reached();
		}
	    }
	  catch(FileException)
	  {
	    gdome_el_unref(el, &exc);
	    throw FileException();
	  }

	}
      gdome_el_unref(el, &exc);
    }

  // Load all the Set members.
  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
      (iSet->second)->loadSetMembers();
}

/** Write to an XML element.

    \return a pointer to the XML element, or NULL on error

    The objects creates an XML element, and writes its persistent data
    to it.  For example, the buffer is saved, but the cursor positon
    is not.
*/
GdomeElement *
Project::writeXML(GdomeDocument *xmlDoc)
{
  GdomeElement *xml;
  return(xml);
}

/** Read from an XML element.

    \param xml is the XML element containing the object data
*/
bool
Project::readXML(GdomeElement *xml)
{
return(true);
}

/** Create the main Project window.

    \param parent is the parent window.  This must be a Gnome
    application window.

    \return pointer to window widget.

    This method should be invoked only once for each object.

*/
void
Project::createWindow(GtkWidget *parent)
{
  GtkWidget *frame;
  GtkWidget *status;

  GtkWidget *docViewParent; // Parent of Document tree view widget
  GtkWidget *catViewParent; // Parent of Category tree view widget
  GtkWidget *setViewParent; // Parent of Set tree view widget

  GtkTreeSelection *selection;

  GtkWidget *scrolledView;
  GtkWidget *hPaneLeft, *hPaneRight;
/***********************************************************************/
  // To hold lists/trees.
  docViewParent = gtk_vbox_new(FALSE, 5);
  catViewParent = gtk_vbox_new(FALSE, 5);
  setViewParent = gtk_vbox_new(FALSE, 5);

  // Labels for the view widgets.
  gtk_box_pack_start(GTK_BOX(docViewParent),
		     gtk_label_new(_("Documents")),
		     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(catViewParent),
		     gtk_label_new(_("Categories")),
		     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(setViewParent),
		     gtk_label_new(_("Sets")),
		     FALSE, FALSE, 0);

  docStore = gtk_tree_store_new(4,
				G_TYPE_STRING,
				G_TYPE_STRING,
				G_TYPE_STRING,
				G_TYPE_POINTER);
  docView = createTreeView(docStore);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(docView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(docView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), docView);
  gtk_box_pack_start(GTK_BOX(docViewParent), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::DOCUMENT);

  catStore = gtk_tree_store_new(QualResObject::N_COLUMNS,
				G_TYPE_STRING,
				G_TYPE_STRING,
				G_TYPE_STRING,
				G_TYPE_POINTER,
				GDK_TYPE_COLOR,
				GDK_TYPE_COLOR,
				G_TYPE_STRING,
				G_TYPE_UINT,
				G_TYPE_BOOLEAN);
  catView = createCatTreeView(catStore);
  addNDocsColumn(catView, NDOCS_COLUMN_NUMBER);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(catView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(catView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), catView);
  gtk_box_pack_start(GTK_BOX(catViewParent), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::CATEGORY);

  setStore = gtk_tree_store_new(4,
				G_TYPE_STRING,
				G_TYPE_STRING,
				G_TYPE_STRING,
				G_TYPE_POINTER);
  setView = createTreeView(setStore);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(setView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(setView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), setView);
  gtk_box_pack_start(GTK_BOX(setViewParent), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::SET);

  // Pack lists.
  hPaneLeft = gtk_hpaned_new();
  gtk_paned_pack1(GTK_PANED(hPaneLeft), docViewParent, FALSE, TRUE);
  gtk_paned_pack2(GTK_PANED(hPaneLeft), catViewParent, TRUE, TRUE);
  gtk_paned_set_position(GTK_PANED(hPaneLeft), QUALRES_XSIZE / 3);

  hPaneRight = gtk_hpaned_new();
  gtk_paned_pack1(GTK_PANED(hPaneRight), hPaneLeft, FALSE, TRUE);
  gtk_paned_pack2(GTK_PANED(hPaneRight), setViewParent, TRUE, TRUE);
  gtk_paned_set_position(GTK_PANED(hPaneRight), 2 * QUALRES_XSIZE / 3);

  // Create the toplevel window, menus.
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

  // Add to parent.
  gnome_app_set_contents(GNOME_APP(parent), frame);
  status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);
  gnome_app_set_statusbar(GNOME_APP(parent), status);
  gnome_app_create_menus_with_data(GNOME_APP(parent), fullMenu, this);
  gnome_app_install_menu_hints(GNOME_APP(parent), fullMenu);

  // Add list widgets.
  gtk_container_add(GTK_CONTAINER(frame), hPaneRight);

  // Save the application window pointer.
  mainWindow = parent;

  // Create the search window.
  createSearchWindow(mainWindow);

  // Create the Memo/Xref window.
  createMemoXrefWindow(mainWindow);

  // Create the Narrative Analysis window.
  createNarrativeWindow(mainWindow);

  // Create the Document Filter window.

  // The mergeCat window is created when requested.

  // Create the popup menu widgets.
  popupDocMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupDocMenu),
				docMenu,
				NULL,
				FALSE,
				0,
				this);

  popupCatMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupCatMenu),
				catMenu,
				NULL,
				FALSE,
				0,
				this);
  popupSetMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupSetMenu),
				setMenu,
				NULL,
				FALSE,
				0,
				this);
}

/** Create the Search window.

    \param parent is the parent window

 */
void
Project::createSearchWindow(GtkWidget *parent)
{
  GtkTreeSelection *selection;
  GtkWidget *vbox, *hbox, *view, *frame;
  GtkWidget *outerVbox;
  GtkWidget *button, *scrolledView;
  GtkWidget *menu, *menuItem;
  GtkWidget *hPaneLeft, *hPaneRight;
  string sProp;
/***********************************************************************/
  // Create widgets.
  searchWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(searchWindow), "delete-event",
		     GTK_SIGNAL_FUNC(hide_cb), NULL);
      
  gtk_window_set_default_size(GTK_WINDOW(searchWindow),
			      SEARCH_XSIZE,
			      SEARCH_YSIZE);
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

  gtk_window_set_title(GTK_WINDOW(searchWindow), "Search");

  // Create the views for the search trees.
  searchDocView = createTreeView(docStore);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(searchDocView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);

  searchCatView = createCatTreeView(catStore);
  addNDocsColumn(searchCatView, NDOCS_COLUMN_NUMBER);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(searchCatView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);

  searchSetView = createTreeView(setStore);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(searchSetView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);

  // Pack the views.
  hPaneLeft = gtk_hpaned_new();
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Select Documents to search")),
		     FALSE, FALSE, 0);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), searchDocView);
  gtk_box_pack_start(GTK_BOX(vbox),
		     scrolledView,
		     TRUE, TRUE, 0);
  gtk_paned_pack1(GTK_PANED(hPaneLeft), vbox, FALSE, TRUE);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Select Categories to search")),
		     FALSE, FALSE, 0);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), searchCatView);
  gtk_box_pack_start(GTK_BOX(vbox),
		     scrolledView,
		     TRUE, TRUE, 0);
  gtk_paned_pack2(GTK_PANED(hPaneLeft), vbox, TRUE, TRUE);
  gtk_paned_set_position(GTK_PANED(hPaneLeft), SEARCH_XSIZE / 3);

  hPaneRight = gtk_hpaned_new();
  gtk_paned_pack1(GTK_PANED(hPaneRight), hPaneLeft, FALSE, TRUE);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Select Sets to search")),
		     FALSE, FALSE, 0);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), searchSetView);
  gtk_box_pack_start(GTK_BOX(vbox),
		     scrolledView,
		     TRUE, TRUE, 0);
  gtk_paned_pack2(GTK_PANED(hPaneRight), vbox, TRUE, TRUE);
  gtk_paned_set_position(GTK_PANED(hPaneRight), 2 * SEARCH_XSIZE / 3);

  // Pack these in the outermost vbox.
  outerVbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(outerVbox), hPaneRight, TRUE, TRUE, 0);

  // Radio button menu to specify the search type.
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox),
		     gtk_label_new(_("Store ")),
		     FALSE, FALSE, 0);

  menu = gtk_menu_new();
  menuItem = gtk_menu_item_new_with_label("text matching");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
  menuItem = gtk_menu_item_new_with_label("word containing");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
  menuItem = gtk_menu_item_new_with_label("sentence containing");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
  menuItem = gtk_menu_item_new_with_label
    ("sentence (with context) containing");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
  menuItem = gtk_menu_item_new_with_label("paragraph containing");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
  menuItem = gtk_menu_item_new_with_label("entire buffer containing");
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
  searchTypeOptionMenu = gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(searchTypeOptionMenu), menu);
  gtk_box_pack_start(GTK_BOX(hbox),
		     searchTypeOptionMenu,
		     FALSE, FALSE, 0);

  // Create search string entry.
  searchText = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox),
		     searchText,
		     TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox),
		     gtk_label_new(_(" into Category ")),
		     FALSE, FALSE, 0);

  // Create destination Category entry and selection button.
  if (searchDestCat != NULL)
    sProp = searchDestCat->getPrettyName();
  else
    sProp = "";

  searchCatBuffer = gtk_text_buffer_new(tagTable);
  view = gtk_text_view_new_with_buffer(searchCatBuffer);

  gtk_box_pack_start(GTK_BOX(hbox),
		     view,
		     FALSE, FALSE, 0);

  button = gtk_button_new_with_label("Select Category");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(selectSearchCategory_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(outerVbox), hbox, FALSE, FALSE, 0);

  // Create buttons.
  hbox = gtk_hbox_new(FALSE, 5);
  // Search.
  button = gtk_button_new_with_label(_("Search"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(search_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);
  // Close window.
  button = gtk_button_new_with_label(_("Close"));
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(hideSearchWindow_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(outerVbox), hbox, FALSE, FALSE, 0);

  // Add the widgets.
  gtk_container_add(GTK_CONTAINER(frame), outerVbox);
  gtk_container_add(GTK_CONTAINER(searchWindow), frame);
}

/** Create the window to merge Categories.

    \param parent is the parent window

 */
void
Project::createMergeCatWindow(GtkWidget *parent)
{
  GtkTreeSelection *selection;
  GtkWidget *vbox, *hbox, *view, *frame;
  GtkWidget *outerVbox;
  GtkWidget *button, *scrolledViewLeft, *scrolledViewRight;
  GtkWidget *menu, *menuItem;
  GtkWidget *hPane;
  GtkAdjustment *adjustment;
  string sProp;
/***********************************************************************/
  // Create widgets.
  mergeCatWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(mergeCatWindow), "delete-event",
		     GTK_SIGNAL_FUNC(hide_cb), NULL);
      
  gtk_window_set_default_size(GTK_WINDOW(mergeCatWindow),
			      MERGECAT_XSIZE,
			      MERGECAT_YSIZE);
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

  gtk_window_set_title(GTK_WINDOW(mergeCatWindow), "Merge Categories");

  // Create the views for the search trees.
  mergeCatMergeView = createCatTreeView(catStore);
  addNDocsColumn(mergeCatMergeView, NDOCS_COLUMN_NUMBER);
  addCheckColumn(mergeCatMergeView, MERGE_CHECK_COLUMN_NUMBER, "Merge");
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mergeCatMergeView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_NONE);

  mergeCatDestView = createCatTreeView(catStore);
  addNDocsColumn(mergeCatDestView, NDOCS_COLUMN_NUMBER);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mergeCatDestView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(mergeCatDestView),
		   "button-release-event",
		   G_CALLBACK(mergeCatDestViewButtonPress_cb),
		   this);
  g_signal_connect(G_OBJECT(mergeCatDestView),
		   "cursor-changed",
		   G_CALLBACK(mergeCatDestListSelect_cb),
		   this);

  // Pack the views.
  hPane = gtk_hpaned_new();
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Select Categories to merge")),
		     FALSE, FALSE, 0);
  scrolledViewLeft = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledViewLeft),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledViewLeft), mergeCatMergeView);
  gtk_box_pack_start(GTK_BOX(vbox),
		     scrolledViewLeft,
		     TRUE, TRUE, 0);
  gtk_paned_pack1(GTK_PANED(hPane), vbox, FALSE, TRUE);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Select destination Category")),
		     FALSE, FALSE, 0);
  scrolledViewRight = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledViewRight),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledViewRight), mergeCatDestView);
  gtk_box_pack_start(GTK_BOX(vbox),
		     scrolledViewRight,
		     TRUE, TRUE, 0);
  gtk_paned_pack2(GTK_PANED(hPane), vbox, TRUE, TRUE);
  gtk_paned_set_position(GTK_PANED(hPane), MERGECAT_XSIZE / 2);

  // Pack these in the outermost vbox.
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hPane, TRUE, TRUE, 0);

  // Display the selected destination Category.
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox),
		     gtk_label_new(_("Destination Category:  ")),
		     FALSE, FALSE, 0);

  mergeCatDestBuffer = gtk_text_buffer_new(tagTable);
  view = gtk_text_view_new_with_buffer(mergeCatDestBuffer);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(view),
			     FALSE);
  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view),
				   FALSE);

  gtk_box_pack_start(GTK_BOX(hbox),
		     view,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  // Create buttons.
  hbox = gtk_hbox_new(FALSE, 5);

  // Close window.
  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  g_signal_connect(G_OBJECT(button),
		   "clicked",
		   G_CALLBACK(hideData_cb),
		   mergeCatWindow);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  // Merge Categories.
  button = gtk_button_new_with_label("Merge");
  g_signal_connect(G_OBJECT(button),
		   "clicked",
		   G_CALLBACK(mergeCatMerge_cb),
		   this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  // Add the widgets.
  gtk_container_add(GTK_CONTAINER(frame), vbox);
  gtk_container_add(GTK_CONTAINER(mergeCatWindow), frame);

  // Scroll all the way to the left.
  gtk_widget_show_all(mergeCatWindow);
  adjustment =
    gtk_scrolled_window_get_hadjustment
    (GTK_SCROLLED_WINDOW(scrolledViewLeft));
  gtk_adjustment_set_value(adjustment, adjustment->lower);
  gtk_adjustment_value_changed(adjustment);

  adjustment =
    gtk_scrolled_window_get_hadjustment
    (GTK_SCROLLED_WINDOW(scrolledViewRight));
  gtk_adjustment_set_value(adjustment, adjustment->lower);
  gtk_adjustment_value_changed(adjustment);
}

/** Select a destination Category for search results.
*/
void
Project::selectSearchCategoryList()
{
  // Variables used in creating the window.
  GtkWidget *vbox, *scrolledView;
  gint result;

  // Create the Category selection window.
  if (selectSearchCatWindow == NULL)
    {
      selectSearchCatWindow =
	gtk_dialog_new_with_buttons("Search destination Category",
				    GTK_WINDOW(mainWindow),
				    GTK_DIALOG_MODAL,
				    GTK_STOCK_OK,
				    GTK_RESPONSE_ACCEPT,
				    GTK_STOCK_CANCEL,
				    GTK_RESPONSE_REJECT,
				    NULL);
      gtk_window_set_default_size(GTK_WINDOW(selectSearchCatWindow),
				  SELECT_COMPONENT_XSIZE,
				  SELECT_COMPONENT_YSIZE);
      // Create the widgets.
      selectSearchCatView = createCatTreeView(catStore);
      addNDocsColumn(selectSearchCatView, NDOCS_COLUMN_NUMBER);
      g_signal_connect(G_OBJECT(selectSearchCatView),
		       "button-press-event",
		       G_CALLBACK(searchCatListButtonPress_cb),
		       this);

      // Pack the vbox.
      vbox = gtk_vbox_new(FALSE, 5);
      gtk_box_pack_start(GTK_BOX(vbox),
			 gtk_label_new(_("Select a Category")),
			 FALSE, FALSE, 0);
      scrolledView = gtk_scrolled_window_new(NULL, NULL);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				     GTK_POLICY_AUTOMATIC,
				     GTK_POLICY_AUTOMATIC);
      gtk_container_add(GTK_CONTAINER(scrolledView), selectSearchCatView);
      gtk_box_pack_start(GTK_BOX(vbox),
			 scrolledView,
			 TRUE, TRUE, 0);
				       
      // Add the view to the selection widget.
      gtk_container_add(GTK_CONTAINER(GTK_DIALOG(selectSearchCatWindow)->vbox),
			vbox);
    }

  // Show the selection window.
  gtk_widget_show_all(selectSearchCatWindow);
  result = gtk_dialog_run(GTK_DIALOG(selectSearchCatWindow));

  // Load the new category name.
  if (result == GTK_RESPONSE_ACCEPT)
      loadSearchCategoryFromList();

  gtk_widget_hide(selectSearchCatWindow);
}

/** Load the search destination Category from the selection.

    \param view is the tree view with the selected Category

 */
void
Project::loadSearchCategoryFromList()
{
  GtkTextIter start, end;
  GtkTextTag *tag;
  gchar *fontName;

  // Variables used in getting the selection.
  GtkTreeSelection *selection;
  gchar *name;
  GtkTreeIter iter;

  // Get the selected Category.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(selectSearchCatView));

  // If there is a selection...
  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
    {
      // Get the Category name.
      gtk_tree_model_get(GTK_TREE_MODEL(catStore),
			 &iter,
			 QualResObject::FULL_NAME, &name,
			 -1);

      // Load the Category and its name.
      searchDestCat = catMap[name];
      g_free(name);

      tag = searchDestCat->getTag();
      gtk_text_buffer_set_text(searchCatBuffer,
			       (searchDestCat->getPrettyName()).c_str(),
			       -1);
      gtk_text_buffer_get_bounds(searchCatBuffer,
				 &start,
				 &end);
      gtk_text_buffer_apply_tag(searchCatBuffer,
				tag,
				&start,
				&end);

      // Try to make the changes visible.  This is a workaround.
      g_object_get((gpointer) tag,
		   "font",
		   &fontName,
		   NULL);
      g_object_set((gpointer) tag,
		   "font",
		   fontName,
		   NULL);
      g_free(fontName);
    }

  // Hide the Category selection window.
    gtk_widget_hide(selectSearchCatWindow);
}

/** Open the selected Component.

    \param widget is the view widget

*/
void
Project::openSelectedComponent(GtkWidget *widget)
{
  vector<Component *> selectedComps; // Container of Components to open.
  vector<Component*>::iterator iSel;
  GtkTreeSelection *selection;

  // Get the selected Project Component.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));

  selectedComps.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addAddressesFromView,
				      (gpointer) &selectedComps);

  // Return if there was nothing selected.
  if (selectedComps.size() == 0)
    return;

  // Open the Components.
  for (iSel = selectedComps.begin(); iSel != selectedComps.end(); iSel++)
    (*iSel)->showWindow();
}

/** Pop up a Component menu.

    \param widget is the view widget

    \param event is the event associated with the mouse click

*/
void
Project::popupComponentMenu(GtkWidget *widget, GdkEventButton *event)
{
  GtkWidget *menu;

  if (widget == docView)
    menu = popupDocMenu;
  else if (widget == catView)
    menu = popupCatMenu;
  else if (widget == setView)
    menu = popupSetMenu;
  else if (widget == memoView)
    menu = popupMemoMenu;
  else if (widget == xrefView)
    menu = popupXrefMenu;
  else if (widget == narrativeView)
    menu = popupNarrativeMenu;
  else if (widget == frameView)
    menu = popupFrameMenu;
  else
    {
      g_assert_not_reached();
      return;
    }

  // Popup the menu.
  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
		 event->button, event->time);
}

/** Populate the Project Component tree model

    \param type contains the type of the new component

    The model is first cleared, and loaded with the names of the
    Project Components
*/
void
Project::updateTreeStore(QualResObject::Type type)
{
  Category *cat;
  // Pointer to Project Component map.
  map<const string, Component *> *compMap;
  map<const string, Component *>::iterator iComp;
  string compName, compShortName;
  string compDesc;

  // Tree model.
  GtkTreeStore *store;
  GtkTreeIter iter, parentIter;
  stack<GtkTreeIter> iterStack;

  // Depth in hierarchy.
  gint level, newLevel;

  // Index.
  gint i;

  // Point to the store.
  switch (type)
    {
    case QualResObject::DOCUMENT:
      compMap = (map<const string, Component *> *) &docMap;
      store = docStore;
      break;
    case QualResObject::CATEGORY:
      compMap = (map<const string, Component *> *) &catMap;
      store = catStore;
      break;
    case QualResObject::SET:
      compMap = (map<const string, Component *> *) &setMap;
      store = setStore;
      break;
    case QualResObject::MEMO:
      compMap = (map<const string, Component *> *) &memoMap;
      store = memoStore;
      break;
    case QualResObject::XREF:
      compMap = (map<const string, Component *> *) &xrefMap;
      store = xrefStore;
      break;
    case QualResObject::NARRATIVE_DOCUMENT:
      compMap = (map<const string, Component *> *) &narrativeMap;
      store = narrativeStore;
      break;
    case QualResObject::FRAME:
      compMap = (map<const string, Component *> *) &frameMap;
      store = frameStore;
      break;
    default:
      g_assert_not_reached();
      return;
    }

  // Clear the store.
  gtk_tree_store_clear(store);
      
  // Add rows.
  level = 0; // Initially at top level
  for (iComp = compMap->begin(); iComp != compMap->end(); iComp++)
    {
      compName = (iComp->second)->getName();
      compShortName = (iComp->second)->getShortName();
      compDesc = (iComp->second)->getDesc();

      // Find out what level we're at.
      newLevel = 0;
      i = -1;
      while ((i = compName.find(SEP_CHAR, i + 1)) != string::npos)
	newLevel++;

      // Descend one level?
      if (newLevel == level + 1)
	{
	  level = newLevel;
	  iterStack.push(parentIter);
	  // There was a previous Component, which is now the parent
	  // Component.
	  parentIter = iter;
	}

      // Ascend one or more levels?
      while (newLevel < level)
	{
	  parentIter = iterStack.top();
	  iterStack.pop();
	  level--;
	}

      // Store it.
      if (level == 0)
	gtk_tree_store_append(store, &iter, NULL);
      else
	  gtk_tree_store_append(store, &iter, &parentIter);

      gtk_tree_store_set(store, &iter,
			 QualResObject::SHORT_NAME, compShortName.c_str(),
			 QualResObject::DESCRIPTION, compDesc.c_str(),
			 QualResObject::FULL_NAME, compName.c_str(),
			 QualResObject::ADDRESS, iComp->second,
			 -1);

      // If this is a Category, load the attributes.
      if ((type == QualResObject::CATEGORY) ||
	  (type == QualResObject::XREF) ||
	  (type == QualResObject::FRAME))
	{
	  cat = (Category *) iComp->second;
	  gtk_tree_store_set(store, &iter,
			     QualResObject::FG_COLOR, cat->getFgColor(),
			     QualResObject::BG_COLOR, cat->getBgColor(),
			     QualResObject::FONT, (*(cat->getFont())).c_str(),
			     QualResObject::N_DOCS, cat->nDocuments(),
			     -1);
	}
    }
}

/** Change the object's properties.

    \param parent is the parent window.  This parameter is is ignored,
    and can be set to NULL.

 */
bool
Project::changeProperties(GtkWindow *parent)
{
  GtkWidget *hbox, *scrolledView, *view;
  GtkTextIter start, end;
  bool status;
  gint result;

  string sProp;

  // Create the properties window.
  if (propWindow == NULL)
    {
      propWindow = gtk_dialog_new_with_buttons("Properties",
					       GTK_WINDOW(mainWindow),
					       GTK_DIALOG_MODAL,
					       GTK_STOCK_OK,
					       GTK_RESPONSE_ACCEPT,
					       GTK_STOCK_CANCEL,
					       GTK_RESPONSE_REJECT,
					       NULL);
      hbox = gtk_hbox_new(FALSE, 5);

      // Add label and text buffer for the description.
      gtk_box_pack_start(GTK_BOX(hbox),
			 gtk_label_new(_("Project description:  ")),
			 FALSE, FALSE, 0);

      // Create view, and put it into a scrolled window.
      view = gtk_text_view_new();
      scrolledView = gtk_scrolled_window_new(NULL, NULL);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				     GTK_POLICY_AUTOMATIC,
				     GTK_POLICY_AUTOMATIC);
      gtk_container_add(GTK_CONTAINER(scrolledView), view);

      gtk_box_pack_start(GTK_BOX(hbox),
			 scrolledView,
			 TRUE, TRUE, 0);

      gtk_container_add(GTK_CONTAINER(GTK_DIALOG(propWindow)->vbox),
			hbox);

      compDesc = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
    }

  // Load the properties.
  gtk_text_buffer_get_bounds(compDesc, &start, &end);
  gtk_text_buffer_delete(compDesc, &start, &end);
  sProp = getDesc();
  gtk_text_buffer_set_text(compDesc, sProp.c_str(), -1);

  // Show the properties window.
  gtk_widget_show_all(propWindow);
  result = gtk_dialog_run(GTK_DIALOG(propWindow));
  gtk_widget_hide(propWindow);

  // Load new properties.
  if (result == GTK_RESPONSE_ACCEPT)
    {
      gtk_text_buffer_get_bounds(compDesc, &start, &end);
      setDesc(gtk_text_buffer_get_text(compDesc, &start, &end, FALSE));
      status = true;
    }
  else
    status = false;

  return(status);
}

/** Inform the object that a component is about to be deleted.

    \param comp is the component

    This method informs all the Project Components that the component
    is about to be deleted.
*/
void
Project::pendingDelete(QualResObject *comp)
{
  map<const string, Document *>::iterator iDoc;
  map<const string, Category *>::iterator iCat;
  map<const string, Set *>::iterator iSet;
  map<const string, Memo *>::iterator iMemo;
  map<const string, Xref *>::iterator iXref;
  map<const string, NarrativeDocument *>::iterator iNDoc;
  map<const string, Frame *>::iterator iFrame;

  if (comp == mergeCatDestCat)
    setMergeCatDest("");

  for (iDoc = docMap.begin(); iDoc != docMap.end(); iDoc++)
    (iDoc->second)->pendingDelete(comp);

  for (iCat = catMap.begin(); iCat != catMap.end(); iCat++)
    (iCat->second)->pendingDelete(comp);

  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
    (iSet->second)->pendingDelete(comp);

  for (iMemo = memoMap.begin(); iMemo != memoMap.end(); iMemo++)
    (iMemo->second)->pendingDelete(comp);

  for (iXref = xrefMap.begin(); iXref != xrefMap.end(); iXref++)
    (iXref->second)->pendingDelete(comp);

  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    (iNDoc->second)->pendingDelete(comp);

  for (iFrame = frameMap.begin(); iFrame != frameMap.end(); iFrame++)
    (iFrame->second)->pendingDelete(comp);
}

/** Inform the object that a component is about to be cut.

    \param comp is the component

    This method informs all the Project Components that the component
    is about to be cut.
*/
void
Project::pendingCut(QualResObject *comp)
{
  map<const string, Document *>::iterator iDoc;
  map<const string, Category *>::iterator iCat;
  map<const string, Set *>::iterator iSet;
  map<const string, Memo *>::iterator iMemo;
  map<const string, Xref *>::iterator iXref;
  map<const string, NarrativeDocument *>::iterator iNDoc;
  map<const string, Frame *>::iterator iFrame;

  if (comp == mergeCatDestCat)
    setMergeCatDest("");

  for (iDoc = docMap.begin(); iDoc != docMap.end(); iDoc++)
    (iDoc->second)->pendingCut(comp);

  for (iCat = catMap.begin(); iCat != catMap.end(); iCat++)
    (iCat->second)->pendingCut(comp);

  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
    (iSet->second)->pendingCut(comp);

  for (iMemo = memoMap.begin(); iMemo != memoMap.end(); iMemo++)
    (iMemo->second)->pendingCut(comp);

  for (iXref = xrefMap.begin(); iXref != xrefMap.end(); iXref++)
    (iXref->second)->pendingCut(comp);

  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    (iNDoc->second)->pendingCut(comp);

  for (iFrame = frameMap.begin(); iFrame != frameMap.end(); iFrame++)
    (iFrame->second)->pendingCut(comp);
}

/** Inform the object that a Component is about to be Pasted.

    \param comp is the component

    This method informs all the Project Components that the component
    is about to be Pasted.
*/
void
Project::pendingPaste(QualResObject *comp)
{
  map<const string, Document *>::iterator iDoc;
  map<const string, Category *>::iterator iCat;
  map<const string, Set *>::iterator iSet;
  map<const string, Memo *>::iterator iMemo;
  map<const string, Xref *>::iterator iXref;
  map<const string, NarrativeDocument *>::iterator iNDoc;
  map<const string, Frame *>::iterator iFrame;

  for (iDoc = docMap.begin(); iDoc != docMap.end(); iDoc++)
    (iDoc->second)->pendingPaste(comp);

  for (iCat = catMap.begin(); iCat != catMap.end(); iCat++)
    (iCat->second)->pendingPaste(comp);

  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
    (iSet->second)->pendingPaste(comp);

  for (iMemo = memoMap.begin(); iMemo != memoMap.end(); iMemo++)
    (iMemo->second)->pendingPaste(comp);

  for (iXref = xrefMap.begin(); iXref != xrefMap.end(); iXref++)
    (iXref->second)->pendingPaste(comp);

  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    (iNDoc->second)->pendingPaste(comp);

  for (iFrame = frameMap.begin(); iFrame != frameMap.end(); iFrame++)
    (iFrame->second)->pendingPaste(comp);
}

/** Inform the object that a component's properties have changed.

    \param comp is the component

    This method informs all the Project Components that the component's
    properties have changed.
*/
void
Project::propertiesHaveChanged(QualResObject *comp)
{
  map<const string, Document *>::iterator iDoc;
  map<const string, Category *>::iterator iCat;
  map<const string, Set *>::iterator iSet;
  map<const string, Memo *>::iterator iMemo;
  map<const string, Xref *>::iterator iXref;
  map<const string, NarrativeDocument *>::iterator iNDoc;
  map<const string, Frame *>::iterator iFrame;

  for (iDoc = docMap.begin(); iDoc != docMap.end(); iDoc++)
    (iDoc->second)->propertiesHaveChanged(comp);

  for (iCat = catMap.begin(); iCat != catMap.end(); iCat++)
    (iCat->second)->propertiesHaveChanged(comp);

  for (iSet = setMap.begin(); iSet != setMap.end(); iSet++)
    (iSet->second)->propertiesHaveChanged(comp);

  for (iMemo = memoMap.begin(); iMemo != memoMap.end(); iMemo++)
    (iMemo->second)->propertiesHaveChanged(comp);

  for (iXref = xrefMap.begin(); iXref != xrefMap.end(); iXref++)
    (iXref->second)->propertiesHaveChanged(comp);

  for (iNDoc = narrativeMap.begin(); iNDoc != narrativeMap.end(); iNDoc++)
    (iNDoc->second)->propertiesHaveChanged(comp);

  for (iFrame = frameMap.begin(); iFrame != frameMap.end(); iFrame++)
    (iFrame->second)->propertiesHaveChanged(comp);
}

/** Initialize a newly-created component.

    \param comp is the newly-create component

    This method loads comp with the Project Component maps and models.
*/
void
Project::initComponent(Component *comp)
{
  if ((comp->getType() == QualResObject::MEMO) ||
      (comp->getType() == QualResObject::XREF))
    {
      // Memos and Cross-references Cross-reference text.
      ((Component *)comp)->loadMap((map<const string, Component *> *) &memoMap,
				   QualResObject::DOCUMENT);
      ((Component *)comp)->loadMap((map<const string, Component *> *) &xrefMap,
				   QualResObject::CATEGORY);
      ((Component *)comp)->loadMap((map<const string, Component *> *)
				   &emptyMap,
				   QualResObject::SET);

      ((Component *)comp)->loadModel(memoStore, QualResObject::DOCUMENT);
      ((Component *)comp)->loadModel(xrefStore, QualResObject::CATEGORY);
    }
  else if ((comp->getType() == QualResObject::NARRATIVE_DOCUMENT) ||
	   (comp->getType() == QualResObject::FRAME))
    {
      // Narrative Documents and Frames are associated.
      ((Component *)comp)->loadMap((map<const string, Component *> *)
				   &narrativeMap,
				   QualResObject::DOCUMENT);
      ((Component *)comp)->loadMap((map<const string, Component *> *)
				   &frameMap,
				   QualResObject::CATEGORY);
      ((Component *)comp)->loadMap((map<const string, Component *> *)
				   &emptyMap,
				   QualResObject::SET);

      ((Component *)comp)->loadModel(narrativeStore, QualResObject::DOCUMENT);
      ((Component *)comp)->loadModel(frameStore, QualResObject::CATEGORY);
    }
  else
    {
      // Documents and Categories Code text.
      ((Component *)comp)->loadMap((map<const string, Component *> *) &docMap,
				   QualResObject::DOCUMENT);
      ((Component *)comp)->loadMap((map<const string, Component *> *) &catMap,
				   QualResObject::CATEGORY);
      ((Component *)comp)->loadMap((map<const string, Component *> *) &setMap,
				   QualResObject::SET);

      ((Component *)comp)->loadModel(docStore, QualResObject::DOCUMENT);
      ((Component *)comp)->loadModel(catStore, QualResObject::CATEGORY);
      ((Component *)comp)->loadModel(setStore, QualResObject::SET);
    }
}

/** Show the memo window.
 */
void
Project::showMemoXrefWindow()
{
  gtk_widget_show_all(memoXrefWindow);
  gtk_window_present(GTK_WINDOW(memoXrefWindow));
}

/** Hide the memo window.
 */
void
Project::hideMemoXrefWindow()
{
  gtk_widget_hide(memoXrefWindow);
}

/** Show the search window.
 */
void
Project::showSearchWindow()
{
  gtk_widget_show_all(searchWindow);
  gtk_window_present(GTK_WINDOW(searchWindow));
}

/** Hide the search window.
 */
void
Project::hideSearchWindow()
{
  gtk_widget_hide(searchWindow);
}

/** Show the merge Categories window.
 */
void
Project::showMergeCatWindow()
{
  if (mergeCatWindow == NULL)
    createMergeCatWindow(mainWindow);

  gtk_widget_show_all(mergeCatWindow);
  gtk_window_present(GTK_WINDOW(mergeCatWindow));
}

/** Show the Narrative Analysis window.
 */
void
Project::showNarrativeWindow()
{
  gtk_widget_show_all(narrativeWindow);
  gtk_window_present(GTK_WINDOW(narrativeWindow));
}

/** Hide the Narrative Analysis window.
 */
void
Project::hideNarrativeWindow()
{
  gtk_widget_hide(narrativeWindow);
}

/** Callback to merge multiple Categories to be the child of a Category.
 */
gboolean
mergeCatMerge_cb(GtkWidget *widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->mergeCatMerge();

  return(TRUE);
}

/** Callback to respond to multiple clicks on the Parent Category list.
 */
gboolean
mergeCatDestViewButtonPress_cb(GtkWidget *widget,
				GdkEventButton *event,
				gpointer data)
{
  Project *proj = (Project *) data;
  GtkTreeModel *model;
  GtkTreeSelection *selection;
  gchar *name;
  GtkTreeIter iter;

  // Click with button 1.  Choose the selected Category.
  if ((event->button == 1) && (event->type == GDK_BUTTON_RELEASE))
    {
      // Get the selected Category.
      selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));

      // Is there a selection?
      if (!gtk_tree_selection_get_selected(selection, NULL, &iter))
	{
	  // No.  Clear the destination Category.
	  proj->setMergeCatDest("");
	  return(FALSE); // Continue with normal handling.
	}

      // Get the Category name.
      model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
      gtk_tree_model_get(model,
			 &iter,
			 QualResObject::FULL_NAME, &name,
			 -1);

      // Load the Category and its name.
      proj->setMergeCatDest(name);
      g_free(name);
    }

  // Continue with normal handling.
  return(FALSE);
}

/** User clicked on a row to select a parent Category.
 */
void mergeCatDestListSelect_cb(GtkTreeView *treeview, gpointer data)
{
  Project *proj = (Project *) data;

  proj->getAndSetMergeCatDest();
}

/** Perform text search.
 */
void
Project::search()
{
  // Selected Project Components.
  map<const string, string> selectedDocs;
  map<const string, string> selectedCats;
  map<const string, string> selectedSets;

  // List of selected Project Components.
  map<const string, string> selectedNames;
  map<const string, string>::iterator iName;

  // Selection.
  GtkTreeSelection *selection;

  // Search parameters.
  string searchString;
  DocCat::SearchStore searchType;

  // Error message.
  GtkWidget *dialog;
  string dialogMessage;

  searchString = gtk_entry_get_text(GTK_ENTRY(searchText));
  searchType = (DocCat::SearchStore)
    gtk_option_menu_get_history(GTK_OPTION_MENU(searchTypeOptionMenu));

  // Check inputs.
  if ((searchString == "") || (searchDestCat == NULL))
    {
      dialogMessage = "You must first enter text to search for and "
	"select a destination Category";
      dialog = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
				      GTK_DIALOG_DESTROY_WITH_PARENT,
				      GTK_MESSAGE_ERROR,
				      GTK_BUTTONS_CLOSE,
				      dialogMessage.c_str());
      gtk_dialog_run(GTK_DIALOG(dialog));
      gtk_widget_destroy(dialog);
      return;
    }

  // Obtain a list of all selected Documents to search.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(searchDocView));

  // Get the names.
  selectedNames.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addNamesFromView,
				      (gpointer) &selectedNames);

  // Add to the Documents to search.
  for (iName = selectedNames.begin(); iName != selectedNames.end(); iName++)
    selectedDocs[iName->second] = iName->second;

  // Obtain a list of all selected Categories to search.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(searchCatView));

  // Get the names.
  selectedNames.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addNamesFromView,
				      (gpointer) &selectedNames);

  // Add to the Categories to search.
  for (iName = selectedNames.begin(); iName != selectedNames.end(); iName++)
    selectedCats[iName->second] = iName->second;

  // Obtain a list of all selected Sets to search.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(searchSetView));

  // Get the names.
  selectedNames.clear();
  gtk_tree_selection_selected_foreach(selection,
				      addNamesFromView,
				      (gpointer) &selectedNames);

  // Add the Components from each Set.
  for (iName = selectedNames.begin(); iName != selectedNames.end(); iName++)
    {
      setMap[iName->second]->loadMemberNames(&selectedDocs,
					     QualResObject::DOCUMENT);
      setMap[iName->second]->loadMemberNames(&selectedCats,
					     QualResObject::CATEGORY);
    }

  // Search and code found text in each Document.
  for (iName = selectedDocs.begin(); iName != selectedDocs.end(); iName++)
      docMap[iName->second]->codeSearchText(searchString,
					    searchType,
					    searchDestCat);

  // Search and code found text in each Category.
  for (iName = selectedCats.begin(); iName != selectedCats.end(); iName++)
      catMap[iName->second]->codeSearchText(searchString,
					    searchType,
					    searchDestCat);
}

/** Get and load the name of the destination Category for merging.

 */
void
Project::getAndSetMergeCatDest()
{
  GtkTreeSelection *selection;
  gchar *name;
  GtkTreeIter iter;

  // Get the selected Category.
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mergeCatDestView));

  // Is there a selection?
  if (!gtk_tree_selection_get_selected(selection, NULL, &iter))
    {
      // No.  Clear the destination Category.
      setMergeCatDest("");
      return;
    }

  // Get the Category name.
  gtk_tree_model_get(GTK_TREE_MODEL(catStore),
		     &iter,
		     QualResObject::FULL_NAME, &name,
		     -1);

  // Load the Category and its name.
  setMergeCatDest(name);

  g_free(name);
}

/** Load the name of the destination Category for merging.

    \param name is the name of the destination Category

*/
void
Project::setMergeCatDest(string name)
{
  GtkTextIter start, end;
  GtkTextTag *tag;
  gchar *fontName;

  if (name == "")
    {
      mergeCatDestCat = NULL;
      gtk_text_buffer_set_text(mergeCatDestBuffer,
			       "",
			       -1);
      return;
    }

  // Load Category name, and apply Category tag to it.
  mergeCatDestCat = catMap[name];
  tag = mergeCatDestCat->getTag();
  gtk_text_buffer_set_text(mergeCatDestBuffer,
			   (mergeCatDestCat->getPrettyName()).c_str(),
			   -1);
  gtk_text_buffer_get_bounds(mergeCatDestBuffer,
			     &start,
			     &end);
  gtk_text_buffer_apply_tag(mergeCatDestBuffer,
			    tag,
			    &start,
			    &end);

  // Try to make the changes visible.  This is a workaround.
  g_object_get((gpointer) tag,
	       "font",
	       &fontName,
	       NULL);
  g_object_set((gpointer) tag,
	       "font",
	       fontName,
	       NULL);
  g_free(fontName);
}

/** Merge the selected Categories to be the children of the selected Category.

    The parent Category pointer is stored in mergeCatDestCat.

    The name of the merged Category is Merged Category, and the
    description includes the descriptions of all the Categories that
    are merged.

    The Categories that are merged are not changed.

 */
void
Project::mergeCatMerge()
{
  GtkTreeSelection *selection;
  GtkTreeIter destIter, insertIter;
  GtkTreeModel *model;
  GtkTreePath *path;

  Category *cat, *destCat;

  // Container of iterators for selected rows.
  vector<GtkTreeIter> selectedIters;
  vector<GtkTreeIter>::iterator iIter;

  return;
#if 0
  // Popup error box if there's no selection.
  if (mergeCatDestCat == NULL)
    {
      showMessage("You must first select a destination Category",
		  GTK_MESSAGE_ERROR,
		  mainWindow);
      return;
    }

  // Get the checked Categories.
  selectedIters.clear();
  gtk_tree_model_foreach(GTK_TREE_MODEL(catStore),
			 loadCheckedItersFromStore,
			 (gpointer) &selectedIters);

  // Get the Categories to merge.
  mergeCats.clear();
  for (iIter = selectedIters.begin();
       iIter != selectedIters.end();
       iIter++)
    {
      // Get the Category address.
      gtk_tree_model_get(GTK_TREE_MODEL(catStore),
			 iIter,
			 QualResObject::ADDRESS, &cat,
			 -1);

      // Put the Category into the container.
      mergeCats.push_back(cat);
      cerr << "Merging Category " << cat->getPrettyName() << endl;
    }

  // Merge the Categories.
  for (iIter = selectedIters.begin();
       iIter != selectedIters.end();
       iIter++)
    {
      gtk_tree_model_get(GTK_TREE_MODEL(catStore),
			 iIter,
			 QualResObject::ADDRESS, &cat,
			 -1);

      // Inform all Project Components of the deletion.
      pendingDelete(cat);

      // Remove the Category from the Project map.
      catMap.erase(cat->getName());

      // Remove the row.
      gtk_tree_store_remove(catStore, iIter);

      // Add the Coded Text to the destination Category.
      

      // Delete the Category.
      delete cat;
    }
#endif
}

/** Create the Memo/Xref window.

    \param parent is the parent window.

*/
void
Project::createMemoXrefWindow(GtkWidget *parent)
{
  GtkWidget *frame, *status, *vbox;
  GtkTreeSelection *selection;
  GtkWidget *scrolledView;
  GtkWidget *hPaneLeft, *hPaneRight;
/***********************************************************************/
  // Create the toplevel window, menus.
  memoXrefWindow = gnome_app_new("Memos", _("Memos"));
  gtk_window_set_default_size(GTK_WINDOW(memoXrefWindow),
			      MEMO_XREF_XSIZE,
			      MEMO_XREF_YSIZE);

  gtk_window_set_title(GTK_WINDOW(memoXrefWindow), _("Memos"));
  gtk_signal_connect(GTK_OBJECT(memoXrefWindow), "delete-event",
		     GTK_SIGNAL_FUNC(hide_cb), NULL);

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gnome_app_set_contents(GNOME_APP(memoXrefWindow), frame);

  status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);
  gnome_app_set_statusbar(GNOME_APP(memoXrefWindow), status);
  gnome_app_create_menus_with_data(GNOME_APP(memoXrefWindow),
				   memoMemoXrefMenu,
				   this);
  gnome_app_install_menu_hints(GNOME_APP(memoXrefWindow), memoMemoXrefMenu);

  // Memos.
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Memos")),
		     FALSE, FALSE, 0);

  memoStore = gtk_tree_store_new(4,
				 G_TYPE_STRING,
				 G_TYPE_STRING,
				 G_TYPE_STRING,
				 G_TYPE_POINTER);
  memoView = createTreeView(memoStore);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(memoView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(memoView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), memoView);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::MEMO);

  hPaneLeft = gtk_hpaned_new();
  gtk_paned_pack1(GTK_PANED(hPaneLeft), vbox, FALSE, TRUE);

  // Cross-references.
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Cross-references")),
		     FALSE, FALSE, 0);

  xrefStore = gtk_tree_store_new(QualResObject::N_COLUMNS,
				 G_TYPE_STRING,
				 G_TYPE_STRING,
				 G_TYPE_STRING,
				 G_TYPE_POINTER,
				 GDK_TYPE_COLOR,
				 GDK_TYPE_COLOR,
				 G_TYPE_STRING,
				 G_TYPE_UINT,
				 G_TYPE_BOOLEAN);
  xrefView = createCatTreeView(xrefStore);
  addNDocsColumn(xrefView, NDOCS_COLUMN_NUMBER);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(xrefView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(xrefView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), xrefView);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::XREF);

  gtk_paned_pack2(GTK_PANED(hPaneLeft), vbox, TRUE, TRUE);

  gtk_paned_set_position(GTK_PANED(hPaneLeft), MEMO_XREF_XSIZE / 2);

  gtk_container_add(GTK_CONTAINER(frame), hPaneLeft);

  // Create the popup menu widgets.
  popupMemoMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupMemoMenu),
				memoMemoMenu,
				NULL,
				FALSE,
				0,
				this);

  popupXrefMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupXrefMenu),
				memoXrefMenu,
				NULL,
				FALSE,
				0,
				this);
}

/** Create the Narrative Analysis window.

    \param parent is the parent window.

*/
void
Project::createNarrativeWindow(GtkWidget *parent)
{
  GtkWidget *frame, *status, *vbox;
  GtkTreeSelection *selection;
  GtkWidget *scrolledView;
  GtkWidget *hPaneLeft, *hPaneRight;
/***********************************************************************/
  // Create the toplevel window, menus.
  narrativeWindow = gnome_app_new("Narrative Analysis",
				  _("Narrative Analysis"));
  gtk_window_set_default_size(GTK_WINDOW(narrativeWindow),
			      NARRATIVE_FRAME_XSIZE,
			      NARRATIVE_FRAME_YSIZE);

  gtk_window_set_title(GTK_WINDOW(narrativeWindow), _("Narrative Analysis"));
  gtk_signal_connect(GTK_OBJECT(narrativeWindow), "delete-event",
		     GTK_SIGNAL_FUNC(hide_cb), NULL);

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gnome_app_set_contents(GNOME_APP(narrativeWindow), frame);

  status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);
  gnome_app_set_statusbar(GNOME_APP(narrativeWindow), status);
  gnome_app_create_menus_with_data(GNOME_APP(narrativeWindow),
				   narrativeAnalysisMenu,
				   this);
  gnome_app_install_menu_hints(GNOME_APP(narrativeWindow),
			       narrativeAnalysisMenu);

  // Narrative Analysis Documents
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Narrative Analysis Documents")),
		     FALSE, FALSE, 0);

  narrativeStore = gtk_tree_store_new(4,
				      G_TYPE_STRING,
				      G_TYPE_STRING,
				      G_TYPE_STRING,
				      G_TYPE_POINTER);
  narrativeView = createTreeView(narrativeStore);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(narrativeView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(narrativeView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), narrativeView);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::NARRATIVE_DOCUMENT);

  hPaneLeft = gtk_hpaned_new();
  gtk_paned_pack1(GTK_PANED(hPaneLeft), vbox, FALSE, TRUE);

  // Frames.
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_label_new(_("Frames")),
		     FALSE, FALSE, 0);

  frameStore = gtk_tree_store_new(QualResObject::N_COLUMNS,
				  G_TYPE_STRING,
				  G_TYPE_STRING,
				  G_TYPE_STRING,
				  G_TYPE_POINTER,
				  GDK_TYPE_COLOR,
				  GDK_TYPE_COLOR,
				  G_TYPE_STRING,
				  G_TYPE_UINT,
				  G_TYPE_BOOLEAN);
  frameView = createCatTreeView(frameStore);
  addNDocsColumn(frameView, NDOCS_COLUMN_NUMBER);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(frameView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  g_signal_connect(G_OBJECT(frameView),
		   "button-press-event",
		   G_CALLBACK(buttonPress_cb),
		   this);
  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), frameView);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledView, TRUE, TRUE, 0);
  updateTreeStore(QualResObject::FRAME);

  gtk_paned_pack2(GTK_PANED(hPaneLeft), vbox, TRUE, TRUE);

  gtk_paned_set_position(GTK_PANED(hPaneLeft), NARRATIVE_FRAME_XSIZE / 2);

  gtk_container_add(GTK_CONTAINER(frame), hPaneLeft);

  // Create the popup menu widgets.
  popupNarrativeMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupNarrativeMenu),
				narrativeDocumentMenu,
				NULL,
				FALSE,
				0,
				this);

  popupFrameMenu = gtk_menu_new();
  gnome_app_fill_menu_with_data(GTK_MENU_SHELL(popupFrameMenu),
				frameMenu,
				NULL,
				FALSE,
				0,
				this);
}

/** Timeout processing for a Project.

    \return TRUE so that the timeout function is called again.

    This method checks if idle processing has been scheduled, and
    schedules it if it has not been.
*/
gboolean
Project::timeout()
{
  // Schedule the idle processing if necessary.
  if (!idleScheduled)
    {
      idleID = gtk_idle_add(projectIdle_cb,
			    this);
      idleScheduled = TRUE;
    }

  return(TRUE);
}

/** Idle processing for a Project.

    \return FALSE so that the idle function is not called again until
    rescheduled.

    This method makes a new autosave file.  It also sets the
    idleScheduled flag to FALSE so that Project::timeout() will
    reschedule idle processing again.
*/
gboolean
Project::idle()
{
  if (!projectSaved())
    {
      // Save to autosave file.
      autoSave();

      // Restore the changed flag.
      setChanged();
    }  

  // Enable rescheduling by the timeout function.
  idleScheduled = FALSE;
  return(FALSE);
}


/** Determine whether to autorecover a file.

    \param file is the file name

    \param autoSave is the name of the autosave file

    \param parent is the parent window for any window that is created

    \returns true if the file should be autorecovered, false otherwise

    If there is no autosave file, or if the autosave file is older
    than the file, then the file should not be autorecovered.

    If the autosave file is newer than the file, the user is prompted
    for input.
*/
enum Project::AutoRecover
Project::autoRecoverFile(string &file, string &autoSave, GtkWidget *parent)
{
  GtkWidget *dialog;
  struct stat fileStat, autoSaveStat;
  int fileStatus, autoSaveStatus;
  gint result;
  gchar *baseName;
  string message;

  // Does the autosave file exist?
  if (!g_file_test(autoSave.c_str(), G_FILE_TEST_EXISTS))
    return(Project::READ_PROJECT); // No.  Read the Project file.

  // Get the times of both files.
  fileStatus = stat(file.c_str(), &fileStat);
  autoSaveStatus = stat(autoSave.c_str(), &autoSaveStat);

  // If the Project file is newer, there is no autosave option.
  if (fileStat.st_mtime >= autoSaveStat.st_mtime)
    return(Project::READ_PROJECT);

  // There is an autosave option.  Construct the prompt.
  if ((fileStatus != 0) || (autoSaveStatus != 0))
    message = "The autosave file was found, but it might be older than "
      "the Project file.\n\n";
  else
    message = "The autosave file was saved more recently than the "
      "Project file:\n\n";

  baseName = g_path_get_basename(fileName.c_str());
  message += baseName;
  g_free(baseName);
  message += " was last saved on ";

  if (fileStatus == 0)
    message += ctime(&fileStat.st_mtime);
  else
    message += "??????\n";

  message += "It was last autosaved on ";

  if (autoSaveStatus == 0)
    message += ctime(&autoSaveStat.st_mtime);
  else
    message += "??????\n";

  // Pop up the dialog window.
  dialog = gtk_message_dialog_new(GTK_WINDOW(parent),
				  GTK_DIALOG_DESTROY_WITH_PARENT,
				  GTK_MESSAGE_QUESTION,
				  GTK_BUTTONS_NONE,
				  message.c_str());

  gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			 "Read autosave file", Project::READ_AUTOSAVE,
			 "Read Project file", Project::READ_PROJECT,
			 GTK_STOCK_CANCEL, Project::CANCEL_READ,
			 NULL);

  result = gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
   
  switch (result)
    {
    case Project::CANCEL_READ:
      return(Project::CANCEL_READ);
      break;
    case Project::READ_AUTOSAVE:
      return(Project::READ_AUTOSAVE);
      break;
    case Project::READ_PROJECT:
      return(Project::READ_PROJECT);
      break;
    default:
      g_return_val_if_reached(Project::CANCEL_READ);
    }
}

/***********************************************************************/
/* CALLBACKS */
/***********************************************************************/
/** Read a Project from disk.
 */
static void
restoreProject_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->restoreFromFile();
}

/** Save the Project.
 */
static void
saveProject_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->saveToFile();
}

/** Save the Project to a new file.
 */
static void
saveProjectAs_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->saveAsToFile();
}

/** Close the Project.
 */
static void
closeProject_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->closeProject();
}

/** Print the names and descriptions of all the Documents.
 */
static void
printDocumentList_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->printComponentList(QualResObject::DOCUMENT);
}

/** Print all the Documents.
 */
static void
printAllDocuments_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->printAllComponents(QualResObject::DOCUMENT);
}

/** Export the names and descriptions of all the Documents to a file.
 */
static void
exportDocumentList_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->exportComponentList(QualResObject::DOCUMENT);
}

/** Export the names and descriptions of all the Categoriess to a file.
 */
static void
exportCategoryList_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->exportComponentList(QualResObject::CATEGORY);
}

/** Print the names and descriptions of all the Categories.
 */
static void
printCategoryList_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->printComponentList(QualResObject::CATEGORY);
}

/** Print all the Categorys.
 */
static void
printAllCategories_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->printAllComponents(QualResObject::CATEGORY);
}

/** Code/view a Document.
 */
static void
codeViewDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->updateComponent(QualResObject::DOCUMENT);
}

/** Cross-reference/view a Memo.
 */
static void
codeViewMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->updateComponent(QualResObject::MEMO);
}

/** Cross-reference/view a Cross-reference
 */
static void
codeViewXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->updateComponent(QualResObject::XREF);
}

/** Code/view a Category.
 */
static void
codeViewCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->updateComponent(QualResObject::CATEGORY);
}

/** Update/view a Set.
 */
static void
updateViewSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->updateComponent(QualResObject::SET);
}

/** Create a new child Document.
 */
static void
newChildDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::DOCUMENT, QualResObject::CHILD);
}

/** Create a new sibling Document.
 */
static void
newSiblingDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::DOCUMENT, QualResObject::SIBLING);
}

/** Create a new child Memo.
 */
static void
newChildMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::MEMO, QualResObject::CHILD);
}

/** Create a new sibling Memo.
 */
static void
newSiblingMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::MEMO, QualResObject::SIBLING);
}

/** Create a new child Category.
 */
static void
newChildCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::CATEGORY, QualResObject::CHILD);
}

/** Create a new sibling Category.
 */
static void
newSiblingCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::CATEGORY, QualResObject::SIBLING);
}

/** Create a new child Cross-reference.
 */
static void
newChildXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::XREF, QualResObject::CHILD);
}

/** Create a new sibling Cross-reference.
 */
static void
newSiblingXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::XREF, QualResObject::SIBLING);
}

/** Create a new child Set.
 */
static void
newChildSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::SET, QualResObject::CHILD);
}

/** Create a sibling child Set.
 */
static void
newSiblingSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::SET, QualResObject::SIBLING);
}

/** Cut a Document.
 */
static void
cutDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::DOCUMENT, Project::CUT);
}

/** Cut a Memo.
 */
static void
cutMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::MEMO, Project::CUT);
}

/** Cut a Cross-reference.
 */
static void
cutXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::XREF, Project::CUT);
}

/** Cut a Category.
 */
static void
cutCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::CATEGORY, Project::CUT);
}

/** Cut a Set.
 */
static void
cutSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::SET, Project::CUT);
}

/** Delete a Document.
 */
static void
deleteDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::DOCUMENT, Project::DELETE);
}

/** Delete a Memo.
 */
static void
deleteMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::MEMO, Project::DELETE);
}

/** Delete a Cross-reference.
 */
static void
deleteXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::XREF, Project::DELETE);
}

/** Delete a Category.
 */
static void
deleteCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::CATEGORY, Project::DELETE);
}

/** Delete a Set.
 */
static void
deleteSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::SET, Project::DELETE);
}

/** Paste Document as sibling.
 */
static void
pasteSiblingDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::DOCUMENT, QualResObject::SIBLING);
}

/** Paste Memo as sibling.
 */
static void
pasteSiblingMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::MEMO, QualResObject::SIBLING);
}

/** Paste Cross-reference as sibling.
 */
static void
pasteSiblingXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::XREF, QualResObject::SIBLING);
}

/** Paste Document as child.
 */
static void
pasteChildDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::DOCUMENT, QualResObject::CHILD);
}

/** Paste Memo as child.
 */
static void
pasteChildMemo_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::MEMO, QualResObject::CHILD);
}

/** Paste Cross-reference as child.
 */
static void
pasteChildXref_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::XREF, QualResObject::CHILD);
}

/** Paste Category as a sibling.
 */
static void
pasteSiblingCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::CATEGORY, QualResObject::SIBLING);
}

/** Paste Category as a child.
 */
static void
pasteChildCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::CATEGORY, QualResObject::CHILD);
}

/** Paste Set as a sibling.
 */
static void
pasteSiblingSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::SET, QualResObject::SIBLING);
}

/** Paste Set as a child.
 */
static void
pasteChildSet_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::SET, QualResObject::CHILD);
}

/** Change Project properties.
 */
static void
changeProjectProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeProperties(NULL);
}

/** Change Document properties.
 */
static void
changeDocumentProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::DOCUMENT);
}

/** Change Memo properties.
 */
static void
changeMemoProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::MEMO);
}

/** Change Cross-reference properties.
 */
static void
changeXrefProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::XREF);
}

/** Change Category properties.
 */
static void
changeCategoryProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::CATEGORY);
}

/** Change Set properties.
 */
static void
changeSetProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::SET);
}

/** Callback to display info about the application.
 */
static void
about_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  static GtkWidget* dialog = NULL;
  GtkWidget* app;

  app = (GtkWidget*) data;

  if (dialog != NULL) 
    {
      g_assert(GTK_WIDGET_REALIZED(dialog));
      gdk_window_show(dialog->window);
      gdk_window_raise(dialog->window);
    }
  else
    {        
      const gchar *authors[] = {
        "Alban Deniz <adeniz@baker-research.com>",
        NULL
      };

      const gchar *documenters[] = {
        "Carla Deniz",
        NULL
      };

      dialog = gnome_about_new (_(PACKAGE_NAME), VERSION,
                                "(C) 2002 Baker Research, Ltd",
				"Qualitative research",
                                authors,
				documenters,
                                _("No translaters"),
                                NULL);

      gtk_signal_connect(GTK_OBJECT(dialog),
                         "destroy",
                         GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                         &dialog);

      gtk_widget_show_all(dialog);
    }
}

/** Callback to show the Memo window.
 */
static void
showMemoXrefWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->showMemoXrefWindow();
}

/** Callback to hide the Memo window.
 */
static void
hideMemoXrefWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->hideMemoXrefWindow();
}

/** Callback to show the Search window.
 */
static void
showSearchWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->showSearchWindow();
}

/** Callback to hide the Search window.
 */
static void
hideSearchWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->hideSearchWindow();
}

/** Callback to show the move Categories window.
 */
static void
showMergeCatWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->showMergeCatWindow();
}

/** Callback to show the Narrative Analysis window.
 */
static void
showNarrativeWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->showNarrativeWindow();
}

/** Callback to hide the Narrative Analysis window.
 */
static void
hideNarrativeWindow_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->hideNarrativeWindow();
}

/** Callback to select a destination Category for the search.
 */
static void
selectSearchCategory_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->selectSearchCategoryList();
}

/** Callback to perform a search.
 */
static void
search_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->search();
}

/** Callback to respond to multiple clicks on Project Components.
 */
static gboolean
buttonPress_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  Project *proj = (Project *) data;

  // Double click with button 1.  Open the selected Component.
  if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1))
    proj->openSelectedComponent(widget);

  // Pop up menu.
  if (event->button == 3)
    proj->popupComponentMenu(widget, event);

  // Continue with normal handling.
  return(FALSE);
}

/** Callback to respond to clicks on the search Category selection list.

    This callback services double-clicks for the Project search window.
*/
gboolean
searchCatListButtonPress_cb(GtkWidget *widget,
			    GdkEventButton *event,
			    gpointer data)
{
  Project *proj = (Project *) data;

  // Double click with button 1.  Choose the selected Category.
  if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1))
    {
      proj->loadSearchCategoryFromList();
      return(TRUE); // End of handling.
    }

  // Continue with normal handling.
  return(FALSE);
}

// Narrative Analysis callbacks.

/** Analyze Narrative Analysis Documents.
 */
static void
analyzeNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->updateComponent(QualResObject::NARRATIVE_DOCUMENT);
}

/** Create a new sibling Narrative Analysis Document.
 */
static void
newSiblingNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::NARRATIVE_DOCUMENT,
			QualResObject::SIBLING);
}

/** Create a new child Narrative Analysis Document.
 */
static void
newChildNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::NARRATIVE_DOCUMENT,
			QualResObject::CHILD);
}

/** Cut a Narrative Analysis Document.
 */
static void
cutNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::NARRATIVE_DOCUMENT, Project::CUT);
}

/** Delete a Narrative Analysis Document.
 */
static void
deleteNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::NARRATIVE_DOCUMENT, Project::DELETE);
}

/** Paste Narrative Analysis Document as sibling.
 */
static void
pasteSiblingNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::NARRATIVE_DOCUMENT,
		       QualResObject::SIBLING);
}

/** Paste Narrative Analysis Document as child.
 */
static void
pasteChildNarrativeDocument_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::NARRATIVE_DOCUMENT,
		       QualResObject::CHILD);
}

/** Change Narrative Analysis Document properties.
 */
static void
changeNarrativeDocumentProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::NARRATIVE_DOCUMENT);
}

/** Create a new sibling Frame.
 */
static void
newSiblingFrame_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::FRAME,
			QualResObject::SIBLING);
}

/** Create a new child Frame.
 */
static void
newChildFrame_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->createComponent(QualResObject::FRAME,
			QualResObject::CHILD);
}

/** Cut a Frame.
 */
static void
cutFrame_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::FRAME, Project::CUT);
}

/** Delete a Frame.
 */
static void
deleteFrame_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->removeComponent(QualResObject::FRAME, Project::DELETE);
}

/** Paste Frame as sibling.
 */
static void
pasteSiblingFrame_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::FRAME,
		       QualResObject::SIBLING);
}

/** Paste Frame as child.
 */
static void
pasteChildFrame_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->pasteComponent(QualResObject::FRAME,
		       QualResObject::CHILD);
}

/** Change Frame properties.
 */
static void
changeFrameProperties_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  proj->changeComponentProperties(QualResObject::FRAME);
}

/** Callback to destroy the application.
 */
static void
projDestroy_cb(GtkWidget* widget, gpointer data)
{
  Project *proj = (Project *) data;

  // Close any open Project.
  if (proj->closeProject() == Project::CANCEL)
    return;

  gtk_main_quit();
}

/** Timeout callback for the Project.
 */
static gboolean
projectTimeout_cb(gpointer data)
{
  Project *proj = (Project *) data;

  return(proj->timeout());
}

/** Idle callback for the Project.
 */
static gboolean
projectIdle_cb(gpointer data)
{
  Project *proj = (Project *) data;

  return(proj->idle());
}
