/*
 * $Id: Set.h,v 1.3 2003/04/16 14:11:47 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef SET_H
#define SET_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "qualres.h"
#include "Document.h"
#include "Category.h"


/** Class for a Set

    This class implements the methods for a Set.

    All access functions of this and derived classes return a pointer
    to const.  For example, getName returns (const string *), pointing
    to the name field.  The object is responsible for this memory; the
    calling program should never free the memory pointed to by this
    pointer.
*/
class Set : public Component
{
public:
  Set();
  Set(GdomeElement *);
  virtual ~Set();

  virtual void pendingDelete(QualResObject *);
  virtual void pendingCut(QualResObject *);
  virtual void propertiesHaveChanged(QualResObject *);

 // Printing methods.
  virtual gchar *getTextToPrint() const;
  virtual gchar *getLabeledName() const;

  void addNamesAndDescriptions(string &, vector<Component *> &) const;

  void loadMemberNames(map<const string, string> *, QualResObject::Type);

  bool member(Component *);
  virtual GdomeElement *writeXML(GdomeDocument *)
    throw(FileException);
  virtual bool readXML(GdomeElement *)
    throw(FileException);
  void loadSetMembers();
  void setChangedMember(Set *, Component *);
  void insertComponent(QualResObject::Type);
  void removeComponent(QualResObject::Type);
  void removeElement(Component *);

  // GUI methods.
  virtual bool changeProperties(GtkWindow *);
  virtual void createWindow(GtkWidget *);
  virtual void showWindow();
  void openSelectedComponent(GtkWidget *);

 private:
  void init();
  void loadSetMemberStore(QualResObject::Type);
  void loadSetMemberSetStore();
  void loadSetRow(Set *);
  void deleteMemberFromSetStore(Component *);
  void deleteMemberFromSetStoreSet(GtkTreeIter *, Component *);
  void updateComponentsInSetStoreSet(GtkTreeIter *);
  void updateLabelComponents(map<const string, string> *,
			     GtkTreeIter *,
			     gboolean,
			     enum QualResObject::Type,
			     GtkTreeIter *);
  void writeMembersXML(GdomeDocument *, GdomeElement *, QualResObject::Type)
    throw(FileException);

  map<Document *, Document *> memberDocMap;  ///< Member Documents
  map<Category *, Category *> memberCatMap;  ///< Member Categories
  map<Set *, Set *> memberSetMap; ///< Member Sets
  vector<string> memberSetNames; ///< Names of member Sets

  GtkWidget *memberDocView; ///< View widget for Document members
  GtkTreeStore *memberDocStore; ///< Document members tree model
  bool memberDocStoreLoaded; ///< Document store loaded?
  GtkWidget *memberCatView; ///< View widget for Category members
  GtkTreeStore *memberCatStore; ///< Category members tree model
  bool memberCatStoreLoaded; ///< Category store loaded?
  GtkWidget *memberSetView; ///< View widget for Set members
  GtkTreeStore *memberSetStore; ///< Set members tree model
  bool memberSetStoreLoaded; ///< Set store loaded?
};

#endif
