/*
 * $Id: Xref.cc,v 1.5 2003/03/27 03:31:09 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Xref.h"

/** Create empty Cross-reference.
 */
Xref::Xref(GtkTextTagTable *t)
{
  init(t);
}

/** Initialize the object

    \param t is the Project tag table

    This method must be called only once for each object.  A tag
    with the name of the object is created and loaded into the
    Project tag table.
*/
void
Xref::init(GtkTextTagTable *t)
{ 
  objType = QualResObject::XREF;
  objName = "Cross-Reference";
  xmlTag = XREF_TAG;
  projTagTable = t;
  catTag = gtk_text_tag_new(NULL);
  catNameBuffer = NULL;
  createWindow(NULL);
  markupInit();
}

/** Destructor.
 */
Xref::~Xref()
{
}

/** Create the window to code and view a Cross-reference.

    \param parent is the parent window

 */
void
Xref::createWindow(GtkWidget *parent)
{
  /* Menus */
  static GnomeUIInfo fileMenu [] = {
    {GNOME_APP_UI_ITEM,
     N_("Export Category"),
     N_("Export the Category to a file"),
     (gpointer) docCatExport_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CONVERT,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Print preview"),
     N_("Print preview"),
     (gpointer) printPreview_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_PRINT_PREVIEW,
     0, (GdkModifierType) 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Print"),
     N_("Print the Document"),
     (gpointer) print_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_PRINT,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Close window"),
     N_("Close the Category window"),
     (gpointer) close_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GNOME_STOCK_PIXMAP_CLOSE,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_END
  };

  static GnomeUIInfo catMenu [] = {
    {GNOME_APP_UI_ITEM,
     N_("Select Category"),
     N_("Select the Category for Coding Text"),
     (gpointer) selectCodeCategoryList_cb, NULL, NULL,
     GNOME_APP_PIXMAP_NONE,
     NULL,
     0, (GdkModifierType) 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Open Selected Category"),
     N_("Open the Selected Category"),
     (gpointer) codeViewSelectedCat_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_OPEN,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Clear Category"),
     N_("Uncode all the Coded Text in this Category"),
     (gpointer) catClearCodedText_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK,
     GTK_STOCK_CLEAR,
     0, (GdkModifierType) 0, NULL},

    GNOMEUIINFO_END
  };

  static GnomeUIInfo help_menu [] = {
    GNOMEUIINFO_HELP ("Category"),
  
    GNOMEUIINFO_END
  };

  static GnomeUIInfo menu [] = {
    { GNOME_APP_UI_SUBTREE_STOCK, N_("File"), NULL, fileMenu, NULL, NULL,
      (GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL },

    { GNOME_APP_UI_SUBTREE_STOCK, N_("Category"), NULL, catMenu, NULL, NULL,
      (GnomeUIPixmapType) 0, NULL, 0, (GdkModifierType) 0, NULL },

    GNOMEUIINFO_MENU_HELP_TREE(help_menu),
    GNOMEUIINFO_END
  };

  GtkWidget *status, *vbox, *hbox, *frame;
  GtkWidget *button;
  GtkWidget *scrolledView, *view;
  
  string sProp;

  // Create widgets.
  mainWindow = gnome_app_new("Cross-reference", _("Cross-reference"));
  gtk_window_set_default_size(GTK_WINDOW(mainWindow),
			      XREF_XSIZE,
			      XREF_YSIZE);
  gtk_signal_connect(GTK_OBJECT(mainWindow), "enter-notify-event",
		     GTK_SIGNAL_FUNC(categoryUpdate_cb), this);

  status = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);
  frame = gtk_frame_new(NULL);

  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  vbox = gtk_vbox_new(FALSE, 5);
  hbox = gtk_hbox_new(FALSE, 5);

  sProp = getName();
  gtk_window_set_title(GTK_WINDOW(mainWindow), sProp.c_str());
  gtk_signal_connect(GTK_OBJECT(mainWindow), "delete-event",
		     GTK_SIGNAL_FUNC(hide_cb), this);

  button = gtk_button_new_with_label("Select");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(selectCodeCategoryList_cb), this);
  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  selectedCatBuffer = gtk_text_buffer_new(projTagTable);
  view = gtk_text_view_new_with_buffer(selectedCatBuffer);

  gtk_box_pack_start(GTK_BOX(hbox),
		     gtk_label_new(_("Selected Cross-reference:  ")),
		     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox),
		     view,
		     FALSE, FALSE, 0);

  button = gtk_button_new_with_label("(Un)Cross-ref");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(categoryCode_cb), this);

  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  button = gtk_button_new_with_label("UnCross-ref row");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
		     GTK_SIGNAL_FUNC(categoryUncodeRow_cb), this);

  gtk_box_pack_start(GTK_BOX(hbox),
		     button,
		     FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(vbox),
		     hbox,
		     FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox),
		     gtk_hseparator_new(),
		     FALSE, FALSE, 0);

  gtk_text_tag_table_add(projTagTable, catTag);

  codedTextStore = gtk_tree_store_new(NCODED_TEXT_COLUMNS,
				      G_TYPE_STRING,
				      G_TYPE_STRING,
				      G_TYPE_STRING,
				      G_TYPE_STRING,
				      G_TYPE_UINT,
				      G_TYPE_BOOLEAN);

  codedTextView = createCodedTextView(codedTextStore);
  g_signal_connect(G_OBJECT(codedTextView),
		   "button-press-event",
		   G_CALLBACK(categoryButtonPress_cb),
		   this);

  scrolledView = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledView),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledView), codedTextView);

  gtk_box_pack_start(GTK_BOX(vbox), scrolledView, TRUE, TRUE, 0);
  loadCodedTextStore(Category::LOAD);

  // Add to application window.
  gnome_app_set_contents(GNOME_APP(mainWindow), frame);
  gnome_app_set_statusbar(GNOME_APP(mainWindow), status);
  gnome_app_create_menus_with_data(GNOME_APP(mainWindow), menu, this);
  gnome_app_install_menu_hints(GNOME_APP(mainWindow), menu);

  // Add the widgets.
  gtk_container_add(GTK_CONTAINER(frame), vbox);
}

/** Get the Component name labeled with "Cross-reference:  ".

    \returns left header of print output (free with g_free)
**/
gchar *
Xref::getLabeledName() const
{
  return(g_strconcat("Cross-reference:  ", (getPrettyName()).c_str(), NULL));
}

