/*
 * $Id: qualres.cc,v 1.1.1.1 2002/12/31 03:12:45 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the qualres Team.  See the
 * ChangeLog files for a list of changes.
 */

#include "config.h"
#include <string>
#include <gnome.h>
#include "Project.h"

gboolean delete_cb(GtkWidget *, GdkEvent *, gpointer);

// Options array.
static int greet_mode = FALSE;
static char* message  = NULL;
static char* geometry = NULL;

struct poptOption options[] = {
  {
    "greet",
    'g',
    POPT_ARG_NONE,
    &greet_mode,
    0,
    N_("Say hello to specific people listed on the command line"),
    NULL
  },
  { 
    "message",
    'm',
    POPT_ARG_STRING,
    &message,
    0,
    N_("Specify a message other than \"Hello, World!\""),
    N_("MESSAGE")
  },
  { 
    "geometry",
    '\0',
    POPT_ARG_STRING,
    &geometry,
    0,
    N_("Specify the geometry of the main window"),
    N_("GEOMETRY")
  },
  {
    NULL,
    '\0',
    0,
    NULL,
    0,
    NULL,
    NULL
  }
};

int 
main(int argc, char* argv[])
{
  GnomeProgram *program;
  poptContext ctx;
  gchar **args;
  GValue value = { 0, };
  gint i;
  
  Project *proj;
  string projFile;
  GtkWidget *top;

  bindtextdomain(PACKAGE, GNOMELOCALEDIR);  
  textdomain(PACKAGE);

  program = gnome_program_init(PACKAGE, VERSION,
			       LIBGNOMEUI_MODULE,
			       argc, argv,
			       GNOME_PARAM_POPT_TABLE, options,
			       GNOME_PARAM_APP_DATADIR, DATADIR,
			       GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);

  top = gnome_app_new(PACKAGE, _(PACKAGE_NAME));
  gtk_window_set_policy(GTK_WINDOW(top), FALSE, TRUE, FALSE);
  gtk_window_set_default_size(GTK_WINDOW(top), QUALRES_XSIZE, QUALRES_YSIZE);
  gtk_window_set_wmclass(GTK_WINDOW(top), PACKAGE, PACKAGE_NAME);

  // Create Project object.
  proj = new Project(top);

  // Connect signals.
  gtk_signal_connect(GTK_OBJECT(top), "delete-event",
		     GTK_SIGNAL_FUNC(delete_cb), proj);

  // Get command-line file specification.
  g_value_init(&value, G_TYPE_POINTER);
  g_object_get_property(G_OBJECT(program),
			GNOME_PARAM_POPT_CONTEXT,
			&value);
  ctx = (poptContext) g_value_get_pointer(&value);
  g_value_unset(&value);
  args = (gchar **) poptGetArgs(ctx);
  if (args)
    {
      proj->setFileName(args[0]);
      proj->restore();
    }

  gtk_widget_show_all(top);

  gtk_main();

  delete proj;
  return 0;
}

/** Delete signal.
 */
gboolean
delete_cb(GtkWidget* widget, GdkEvent *event, gpointer data)
{
  Project *proj = (Project *) data;

  // Close any open Project.
  if (proj->closeProject() == Project::CANCEL)
    return(TRUE);

  gtk_main_quit();
  return(FALSE);
}
