/*
 * $Id: qualresUtility.h,v 1.14 2003/03/31 22:32:38 adeniz Exp $
 * This file is part of qualres
 *
 * Copyright (C) 2002  Baker Research, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */
 
/*
 * See the AUTHORS file for a list of people on the gedit Team.  See the
 * ChangeLog files for a list of changes.
 */

#ifndef QUALRES_UTILITY_H
#define QUALRES_UTILITY_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <gtk/gtk.h>
#include <gnome.h>
#include <QualResObject.h>

#define NEWLINE_CHAR '\n'
#define NEWLINE_STRING "\n"

#define SEP_CHAR '\n'
#define SEP_STRING "\n"
#define SEP_CHAR_LEN 1
#define PRETTY_SEP_STRING "->"

// Backup file suffix.
#define BACKUP_SUFFIX "~"

// Label for Components in Set window.
#define SET_DOCUMENT_LABEL "Documents--"
#define SET_CATEGORY_LABEL "Categories--"

// Component list print output formatting.  See Category::getTextToPrint
// and Set::getTextToPrint.
#define PRINT_NUMBER_PAD "  "
#define PRINT_ID_PAD PRINT_NUMBER_PAD "  "
#define PRINT_LINE_PAD PRINT_NUMBER_PAD PRINT_ID_PAD "  "
#define PRINT_ID_LINE_PAD "  "
#define PRINT_DESCRIPTION_PAD "  "
#define PRINT_COMPONENT_INDENT "  "

// Export formatting.
#define EXPORT_DESC_PAD "  "
#define EXPORT_SEPARATOR "----------------------------\n\n"

// Project file pattern.
#define PROJECT_FILE_PATTERN "*.qualres"

// Initial application window size.
#define QUALRES_XSIZE 500
#define QUALRES_YSIZE 250

// Initial Document window size.
#define DOCUMENT_XSIZE 450
#define DOCUMENT_YSIZE 350

// Initial Search window size.
#define SEARCH_XSIZE 450
#define SEARCH_YSIZE 350

// Initial Memo/Xref window size.
#define MEMO_XREF_XSIZE 450
#define MEMO_XREF_YSIZE 350

// Initial Memo window size.
#define MEMO_XSIZE 450
#define MEMO_YSIZE 350

// Initial Cross-reference window size.
#define XREF_XSIZE 450
#define XREF_YSIZE 350

// Initial Set window size.
#define SET_XSIZE 450
#define SET_YSIZE 350

// Initial Narrative Analysis window size.
#define NARRATIVE_FRAME_XSIZE 450
#define NARRATIVE_FRAME_YSIZE 350

// Text delimiter for Frameed text.
#define FRAME_TEXT_DELIMITER "\n\n"
#define FRAME_TEXT_DELIMITER_SIZE 2

// Initial size of window to move multiple Categories.
#define MERGECAT_XSIZE 400
#define MERGECAT_YSIZE 300

// Initial Select Component window size.
#define SELECT_COMPONENT_XSIZE 200
#define SELECT_COMPONENT_YSIZE 300

// Number of Documents column number in Category tree view.
#define NDOCS_COLUMN_NUMBER 1

// Column number for Category merge checkbox.
#define MERGE_CHECK_COLUMN_NUMBER 0

// ID delimiter.
#define ID_DELIMITER_STRING ":"

// Font size in Document and Category buffers.
#define FONT_SIZE 8

// Default font for Document and Category.
#define DEFAULT_FONT "sans 10"

// Line number field width in Category display.
#define LINE_NUMBER_WIDTH 5

// Autosave interval (milliseconds)
#define AUTOSAVE_MSEC 10000

// Position of start of Coded Text in the Document window when text in
// a Document is made visible.  Values must be between 0.0 and 1.0
#define DOC_MARK_VISIBLE_X 0.0
#define DOC_MARK_VISIBLE_Y 0.2

// XML tags.
#define PROJECT_TAG "Project"
#define NAME_TAG "NAME"
#define DESCRIPTION_TAG "Desc"
#define DOCUMENT_TAG "Doc"
#define CATEGORY_TAG "Cat"
#define SET_TAG "Set"
#define MEMO_TAG "Memo"
#define XREF_TAG "Xref"
#define NARRATIVE_DOCUMENT_TAG "NDoc"
#define FRAME_TAG "Frm"
#define TEXT_TAG "Text"
#define DOCUMENT_CODED_TEXT_TAG "DCT"
#define CODED_TEXT_TAG "CT"
#define MEMBER_DOCUMENT_TAG "MD"
#define MEMBER_CATEGORY_TAG "MC"
#define MEMBER_SET_TAG "MS"

// XML attributes.
#define PACKAGE_ATTR "PKG"
#define VERSION_ATTR "VER"
#define FONT_ATTR "FONT"
#define FG_RGB_ATTR "FG_RGB"
#define BG_RGB_ATTR "BG_RGB"
#define CODED_TEXT_LIMITS_ATTR "CTL"

void showMessage(string, GtkMessageType, GtkWidget *);
void moveToSentenceBounds(GtkTextIter *, GtkTextIter *, gint);
void moveToStatementBounds(GtkTextIter *, GtkTextIter *);
string getStatementID(GtkTextIter *);
void deleteTreeRow(GtkTreeStore *, GtkTreeIter *);
void addNamesFromView(GtkTreeModel *, GtkTreePath *, GtkTreeIter *, gpointer);
void addAddressesFromView(GtkTreeModel *,
			  GtkTreePath *,
			  GtkTreeIter *,
			  gpointer);
void addItersFromView(GtkTreeModel *, GtkTreePath *, GtkTreeIter *, gpointer);
gboolean loadComponentAddressesFromStore(GtkTreeModel *,
					 GtkTreePath *,
					 GtkTreeIter *,
					 gpointer);
gboolean loadCheckedItersFromStore(GtkTreeModel *,
				   GtkTreePath *,
				   GtkTreeIter *,
				   gpointer);

// Error reporting macros.
#define QUALRES_CRITICAL \
  g_critical("file %s: line %d", __FILE__, __LINE__)


// Callbacks.
void destroy_cb(GtkWidget *, gpointer);
void nothing_cb(GtkWidget*, gpointer);
void close_cb(GtkWidget *, gpointer);
gboolean hide_cb(GtkWidget *, GdkEvent *, gpointer);
gboolean hideData_cb(GtkWidget *, gpointer);
void toggle_cb(GtkCellRendererToggle *, gchar *, gpointer);

#endif
